@@ -56,7 +56,7 @@
  *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: alert.c,v 1.33 2004/02/18 22:31:42 chopp Exp $
+ *  @version \$Id: alert.c,v 1.36 2004/08/15 19:55:40 martinp Exp $
  *  @file
  */
 
@@ -66,7 +66,7 @@
 
 static void copy_mail(Mail_T, Mail_T);
 static void replace_bare_linefeed(Mail_T *);
-static void substitute(Mail_T*, char *name, const char *event);
+static void substitute(Mail_T*, char *, const char *, const char *);
 
 
 /* ------------------------------------------------------------------ Public */
@@ -79,47 +79,42 @@ static void substitute(Mail_T*, char *name, const char *event);
 void handle_alert(Event_T E) {
 
   Service_T s;
-  
+
   ASSERT(E);
 
   s= Event_get_source(E);
-
   if(s->maillist||Run.maillist) {
-
     Mail_T m;
     Mail_T n;
     sigset_t ns, os;
     Mail_T list= NULL;
-
     set_signal_block(&ns, &os);
-    
     /*
      * Build a mail-list with local recipients that has registered interest
      * for this event.
      */
     for(m= s->maillist; m; m= m->next) {
       
-      if(IS_EVENT_SET(m->events, Event_get_id(E)) ||
-	 /* Failed events are always reported */
-	 (Event_get_id(E) == EVENT_FAILED) ) {
+      if(IS_EVENT_SET(m->events, Event_get_id(E))) {
 
 	Mail_T tmp;
 
 	NEW(tmp);
-	
+
 	copy_mail(tmp, m);
 
 	if(Event_get_message(E)) {
 	  tmp->opt_message= xstrdup(Event_get_message(E));
 	}
 
-	substitute(&tmp, s->name, EVENT_DESCRIPTION(E));
-	
+	substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
+          EVENTACTION_DESCRIPTION(E));
+
 	replace_bare_linefeed(&tmp);
-	
+
 	tmp->next= list;
 	list= tmp;
-	
+
 	DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
 
       }
@@ -132,7 +127,7 @@ void handle_alert(Event_T E) {
      * overrides the same recipient events which are registered globaly.
      */
     for(m= Run.maillist; m; m= m->next) {
-      
+
       int skip= FALSE;
 
       for(n= s->maillist; n; n= n->next) {
@@ -142,27 +137,25 @@ void handle_alert(Event_T E) {
 	}
       }
 
-      if(!skip &&
-           (IS_EVENT_SET(m->events, Event_get_id(E)) ||
-	   (Event_get_id(E) == EVENT_FAILED))
-	) {
+      if(!skip && IS_EVENT_SET(m->events, Event_get_id(E))) {
 
 	Mail_T tmp;
 
 	NEW(tmp);
-	
+
 	copy_mail(tmp, m);
 
 	if(Event_get_message(E))
 	  tmp->opt_message= xstrdup(Event_get_message(E));
 
-	substitute(&tmp, s->name, EVENT_DESCRIPTION(E));
-	
+	substitute(&tmp, s->name, EVENT_DESCRIPTION(E),
+          EVENTACTION_DESCRIPTION(E));
+
 	replace_bare_linefeed(&tmp);
-	
+
 	tmp->next= list;
 	list= tmp;
-	
+
 	DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
 
       }
@@ -170,10 +163,10 @@ void handle_alert(Event_T E) {
     }
 
     if(list) {
-      
+
       sendmail(list);
       gc_mail_list(&list);
-      
+
     }
 
     unset_signal_block(&os);
@@ -183,12 +176,13 @@ void handle_alert(Event_T E) {
 }
 
 
-static void substitute(Mail_T *m, char *name, const char *event) {
+static void substitute(Mail_T *m, char *name, const char *event,
+  const char *action) {
 
   char host[STRLEN];
   char *now= get_RFC822date(NULL);
 
-  ASSERT(m && name && event);
+  ASSERT(m && name && event && action);
 
   if(gethostname(host, sizeof(host)) < 0) {
     snprintf(host, STRLEN, "%s", LOCALHOST);
@@ -205,6 +199,8 @@ static void substitute(Mail_T *m, char *name, const char *event) {
   replace_string(&(*m)->message, "$SERVICE", name);
   replace_string(&(*m)->subject, "$EVENT", event);
   replace_string(&(*m)->message, "$EVENT", event);
+  replace_string(&(*m)->subject, "$ACTION", action);
+  replace_string(&(*m)->message, "$ACTION", action);
  
   FREE(now);
 
