@@ -60,7 +60,7 @@
  *  @author Rory Toma, <rory@digeo.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: control.c,v 1.73 2004/02/06 20:02:20 martinp Exp $
+ *  @version \$Id: control.c,v 1.83 2004/08/28 08:31:07 martinp Exp $
  *
  *  @file
  */
@@ -83,98 +83,17 @@ static void  do_depend(Service_T, const char *);
 /* ------------------------------------------------------------------ Public */
 
 
-/**
- * Handle the action for all processes in service list
- * @param action A string describing the action to execute
- */
-void control(const char *action) {
-
-  Service_T s;
-
-  ASSERT(action);
-
-  for(s= servicelist; s; s= s->next) {
-    if(s->visited)
-	continue;
-    if(exist_daemon()) {
-      d_check_service(s->name, action);
-    } else {
-      check_service(s->name, action);
-    }
-  }
-  
-  reset_depend();
-
-}
-
-
-/*
- * Handle the action for all processes in a group
- * @param G group name
- * @param action A string describing the action to execute
- */
-void control_group(const char *G, const char *action) {
-
-  Service_T s;
-
-  ASSERT(G);
-  ASSERT(action);
-
-  for(s= servicelist; s; s= s->next) {
-    if(s->visited)
-	continue;
-    if(IS(s->group, G)) {
-      if(exist_daemon()) {
-	d_check_service(s->name, action);
-      } else {
-	check_service(s->name, action);
-      }
-    }
-  }
-
-  reset_depend();
- 
-}
-
-
-/*
- * Handle the action for a specified process
- * @param P process name
- * @param action A string describing the action to execute
- */
-void control_process(const char *P, const char *action) {
-
-  Service_T s;
-
-  ASSERT(P);
-  ASSERT(action);
-
-  if(NULL==(s= get_service(P))) {
-    log("%s: Cannot %s program '%s' -- not found in %s\n",
-          prog, action, P, Run.controlfile);
-    return;
-  }
-
-  if(exist_daemon()) {
-    d_check_service(P, action);
-  } else {
-    check_service(P, action);
-  }
- 
-}
-
-
 /**
  * Pass on to methods in http/cervlet.c to start/stop services
- * @param P A service name as stated in the config file
+ * @param S A service name as stated in the config file
  * @param action A string describing the action to execute
  */
-void d_check_service(const char *P, const char *action) {
+void control_service_daemon(const char *S, const char *action) {
 
   Socket_T s;
   char *auth= get_basic_authentication_header();
 
-  ASSERT(P);
+  ASSERT(S);
   ASSERT(action);
 
 
@@ -189,7 +108,7 @@ void d_check_service(const char *P, const char *action) {
     
   } else {
 
-    socket_print(s, "GET /%s?action=%s HTTP/1.0\r\n%s\r\n", P, action, auth);
+    socket_print(s, "GET /%s?action=%s HTTP/1.0\r\n%s\r\n", S, action, auth);
     socket_free(&s);
       
   }
@@ -202,33 +121,33 @@ void d_check_service(const char *P, const char *action) {
 
 /**
  * Check to see if we should try to start/stop service
- * @param P A service name as stated in the config file
+ * @param S A service name as stated in the config file
  * @param action A string describing the action to execute
  * @return TRUE if the service was handled successfully otherwise FALSE
  */
-void check_service(const char *P, const char *action) {
+void control_service(const char *S, const char *action) {
 
   Service_T s= NULL;
 
-  ASSERT(P);
+  ASSERT(S);
   ASSERT(action);
 
-  if(NULL==(s= get_service(P))) {
-    log("%s: Cannot %s program '%s' -- not found in %s\n",
-          prog, action, P, Run.controlfile);
+  if(!(s= get_service(S))) {
+    log("%s: Cannot %s service '%s' -- not found in %s\n",
+          prog, action, S, Run.controlfile);
     return;
   }
 
   if(IS(action, "start")) {
     
     if(s->type==TYPE_PROCESS && is_process_running(s)) {
-      DEBUG("%s: Process already running -- process %s\n", prog, P);
+      DEBUG("%s: Process already running -- process %s\n", prog, S);
       monitor_set(s);
       return;
     }
     
     if(s->type==TYPE_PROCESS && !s->start) {
-      DEBUG("%s: Start method not defined -- process %s\n", prog, P);
+      DEBUG("%s: Start method not defined -- process %s\n", prog, S);
       monitor_set(s);
       return;
     }
@@ -240,7 +159,7 @@ void check_service(const char *P, const char *action) {
   } else if(IS(action, "stop")) {
     
     if(s->type==TYPE_PROCESS && !s->stop) {
-      DEBUG("%s: Stop method not defined -- process %s\n", prog, P);
+      DEBUG("%s: Stop method not defined -- process %s\n", prog, S);
       monitor_unset(s);
       return;
     }
@@ -251,11 +170,11 @@ void check_service(const char *P, const char *action) {
   } else if(IS(action, "restart")) {
     
     if(s->type==TYPE_PROCESS && (!s->start || !s->stop)) {
-      DEBUG("%s: Start or stop method not defined -- process %s\n", prog, P);
+      DEBUG("%s: Start or stop method not defined -- process %s\n", prog, S);
       monitor_set(s);
       return;
     } else {
-      log("Trying to restart '%s'\n", s->name);
+      log("'%s' trying to restart\n", s->name);
     }
     
     do_depend(s, "stop");
@@ -334,7 +253,7 @@ static void do_start(Service_T s) {
     int status;
     pthread_t thread;
 
-    log("start: (%s) %s\n", s->name, s->start->arg[0]);
+    log("'%s' start: %s\n", s->name, s->start->arg[0]);
     spawn(s, s->start, "Started");
     if(s->type==TYPE_PROCESS) {
       /* We only wait for a process type, other service types does not
@@ -344,6 +263,9 @@ static void do_start(Service_T s) {
       END_LOCK;
       status= pthread_create(&thread, NULL, wait_start, s);
       if(status != 0) {
+        LOCK(Run.mutex)
+          Run.wait_start--;
+        END_LOCK;
 	log("Warning: Failed to create the start controller thread. "
 	    "Thread error -- %s.\n", strerror(status));
       }
@@ -373,11 +295,11 @@ static int do_stop(Service_T s) {
 
   if(s->type==TYPE_PROCESS) {
     /* Reset the proc info object in case of a later restart */
-    memset(s->procinfo, 0, sizeof *(s->procinfo));
+    reset_procinfo(s);
   }
 
   if(s->stop && (s->type!=TYPE_PROCESS || is_process_running(s))) {
-    log("stop: (%s) %s\n", s->name, s->stop->arg[0]);
+    log("'%s' stop: %s\n", s->name, s->stop->arg[0]);
     spawn(s, s->stop, "Stopped");
 
     if(s->type==TYPE_PROCESS) {
@@ -441,7 +363,7 @@ static void do_unmonitor(Service_T s) {
    
   if(s->type==TYPE_PROCESS) {
     /* Reset the proc info object in case of a later restart */
-    memset(s->procinfo, 0, sizeof *(s->procinfo));
+    reset_procinfo(s);
   }
   
 }
@@ -455,10 +377,8 @@ static void monitor_set(Service_T s) {
 
   ASSERT(s);
 
-  if(!s->do_monitor) {
-    LOCK(Run.mutex)
-      s->do_monitor= TRUE;
-    END_LOCK;
+  if(!s->monitor) {
+    s->monitor= MONITOR_INIT;
     DEBUG("Monitoring enabled -- service %s\n", s->name);
   }
 
@@ -473,10 +393,8 @@ static void monitor_unset(Service_T s) {
 
   ASSERT(s);
 
-  if(s->do_monitor) {
-    LOCK(Run.mutex)
-      s->do_monitor= FALSE;
-    END_LOCK;
+  if(s->monitor) {
+    s->monitor= MONITOR_NOT;
     DEBUG("Monitoring disabled -- service %s\n", s->name);
   }
 
@@ -530,7 +448,7 @@ static void do_depend(Service_T s, const char *action) {
     
 
 /*
- * This function runs in it's own thread and waits for the service to
+ * This function runs in its own thread and waits for the service to
  * start running. If the service did not start a failed event is
  * posted to notify the user.
  * @param service A Service to wait for
@@ -551,7 +469,11 @@ static void *wait_start(void *service) {
   }
   
   if(!is_process_running(s)) {
-    Event_post(s, EVENT_FAILED, "Failed to start '%s'\n", s->name);
+    Event_post(s, EVENT_EXEC, TRUE, s->action_EXEC,
+      "'%s' failed to start", s->name);
+  } else {
+    Event_post(s, EVENT_EXEC, FALSE, s->action_EXEC,
+      "'%s' started", s->name);
   }
 
   LOCK(Run.mutex)
@@ -585,8 +507,12 @@ static int wait_stop(Service_T s) {
   }
 
   if(is_process_running(s)) {
-    Event_post(s, EVENT_FAILED, "Failed to stop '%s'\n", s->name);
+    Event_post(s, EVENT_EXEC, TRUE, s->action_EXEC,
+      "'%s' failed to stop", s->name);
     return FALSE;
+  } else {
+    Event_post(s, EVENT_EXEC, FALSE, s->action_EXEC,
+      "'%s' stopped", s->name);
   }
 
   return TRUE;
