@@ -20,8 +20,8 @@
 #ifndef MONIT_DEVICE_H
 #define MONIT_DEVICE_H
 
-char *DeviceInfo_Path(DeviceInfo_T, char *);
-int   DeviceInfo_Usage(DeviceInfo_T, char *);
+char *device_path(Info_T, char *);
+int   device_usage(Info_T, char *);
 
 #endif
 
