@@ -20,8 +20,8 @@
 #ifndef MONIT_DEVICE_SYSDEP_H
 #define MONIT_DEVICE_SYSDEP_H
 
-char *DeviceInfo_MountPoint_sysdep(DeviceInfo_T, char *);
-int   DeviceInfo_Usage_sysdep(DeviceInfo_T);
+char *device_mountpoint_sysdep(Info_T, char *);
+int   device_usage_sysdep(Info_T);
 
 #endif
 
