@@ -23,7 +23,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_DARWIN.c,v 1.9 2004/02/29 22:24:44 martinp Exp $
+ *  @version \$Id: sysdep_DARWIN.c,v 1.11 2004/08/09 19:57:52 martinp Exp $
  *
  *  @file
  */
@@ -58,15 +58,15 @@
  * In the case of success, mountpoint is stored in device information
  * structure for later use.
  *
- * @param devinfo  Information structure where resulting data will be stored
+ * @param inf      Information structure where resulting data will be stored
  * @param blockdev Identifies block special device
  * @return         NULL in the case of failure otherwise mountpoint
  */
-char *DeviceInfo_MountPoint_sysdep(DeviceInfo_T devinfo, char *blockdev) {
+char *device_mountpoint_sysdep(Info_T inf, char *blockdev) {
 
   struct statfs usage;
 
-  ASSERT(devinfo);
+  ASSERT(inf);
   ASSERT(blockdev);
 
   if(statfs(blockdev, &usage) != 0) {
@@ -75,7 +75,7 @@ char *DeviceInfo_MountPoint_sysdep(DeviceInfo_T devinfo, char *blockdev) {
     return NULL;
   }
 
-  return strncpy(devinfo->mntpath, usage.f_mntonname, sizeof(devinfo->mntpath));
+  return strncpy(inf->mntpath, usage.f_mntonname, sizeof(inf->mntpath));
 
 }
 
@@ -84,27 +84,27 @@ char *DeviceInfo_MountPoint_sysdep(DeviceInfo_T devinfo, char *blockdev) {
  * MacOS X filesystem usage statistics. In the case of success result is stored in
  * given information structure.
  *
- * @param devinfo Information structure where resulting data will be stored
- * @return        TRUE if informations were succesfully read otherwise FALSE
+ * @param inf Information structure where resulting data will be stored
+ * @return    TRUE if informations were succesfully read otherwise FALSE
  */
-int DeviceInfo_Usage_sysdep(DeviceInfo_T devinfo) {
+int device_usage_sysdep(Info_T inf) {
 
   struct statfs usage;
 
-  ASSERT(devinfo);
+  ASSERT(inf);
 
-  if(statfs(devinfo->mntpath, &usage) != 0) {
+  if(statfs(inf->mntpath, &usage) != 0) {
     log("%s: Error getting usage statistics for device '%s' -- %s\n",
-        prog, devinfo->mntpath, STRERROR);
+        prog, inf->mntpath, STRERROR);
     return FALSE;
   }
 
-  devinfo->f_bsize=           usage.f_bsize;
-  devinfo->f_blocks=          usage.f_blocks;
-  devinfo->f_blocksfree=      usage.f_bavail;
-  devinfo->f_blocksfreetotal= usage.f_bfree;
-  devinfo->f_files=           usage.f_files;
-  devinfo->f_filesfree=       usage.f_ffree;
+  inf->f_bsize=           usage.f_bsize;
+  inf->f_blocks=          usage.f_blocks;
+  inf->f_blocksfree=      usage.f_bavail;
+  inf->f_blocksfreetotal= usage.f_bfree;
+  inf->f_files=           usage.f_files;
+  inf->f_filesfree=       usage.f_ffree;
 
   return TRUE;
 
