@@ -23,7 +23,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: sysdep_UNKNOWN.c,v 1.6 2004/02/29 22:24:45 martinp Exp $
+ *  @version \$Id: sysdep_UNKNOWN.c,v 1.8 2004/08/09 19:57:52 martinp Exp $
  *
  *  @file
  */
@@ -36,11 +36,11 @@
 /**
  * Unknown OS special block device dummy mountpoint method.
  *
- * @param devinfo  Information structure
+ * @param inf  Information structure
  * @param blockdev Identifies block special device
  * @return         NULL
  */
-char *DeviceInfo_MountPoint_sysdep(DeviceInfo_T devinfo, char *blockdev) {
+char *device_mountpoint_sysdep(Info_T inf, char *blockdev) {
 
   log("%s: Unsupported mounted filesystem information method", prog);
 
@@ -52,10 +52,10 @@ char *DeviceInfo_MountPoint_sysdep(DeviceInfo_T devinfo, char *blockdev) {
 /**
  * Unknown OS filesystem dummy usage statistics.
  *
- * @param devinfo Information structure
+ * @param inf Information structure
  * @return        FALSE
  */
-int DeviceInfo_Usage_sysdep(DeviceInfo_T devinfo) {
+int device_usage_sysdep(Info_T inf) {
 
   log("%s: Unsupported filesystem informations gathering method\n", prog);
 
