@@ -51,12 +51,19 @@
 #include <sys/stat.h>
 #endif
 
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
 #endif
 
 #include "monitor.h"
 
+#ifndef MAXPATHLEN
+#define MAXPATHLEN STRLEN
+#endif
 
 /* Private prototypes */
 static void set_sandbox(void);
@@ -68,7 +75,7 @@ static void set_environment(void);
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: env.c,v 1.24 2004/03/12 16:27:03 martinp Exp $
+ *  @version \$Id: env.c,v 1.27 2004/08/11 23:51:12 rory Exp $
  * 
  *  @file
  */
@@ -88,23 +95,6 @@ void init_env() {
   /* Setup program environment */
   set_environment();
 
-  /* Free program environment on exit */
-  atexit(destroy_env);
-  
-}
-
-
-/**
- * Destroy the program environment
- */
-void destroy_env() {
-  
-  FREE(Run.Env.cwd);
-  FREE(Run.Env.home);
-  FREE(Run.Env.user);
-  FREE(Run.localhostname);
-  FREE(Run.controlfile);
-  
 }
 
 
@@ -188,7 +178,7 @@ static void set_sandbox(void) {
  */
 static void set_environment(void) {
 
-  char cwd[STRLEN];
+  char cwd[MAXPATHLEN];
   struct passwd *pw;
   char localhost[STRLEN];
   
