@@ -22,61 +22,62 @@
 
 #include "monitor.h"
 
-
-#define EVENT_NULL       0x0
-#define EVENT_FAILED     0x01
-#define EVENT_START      0x02
-#define EVENT_STOP       0x04
-#define EVENT_RESTART    0x08
-#define EVENT_CHECKSUM   0x10
-#define EVENT_RESOURCE   0x20
-#define EVENT_TIMEOUT    0x40
-#define EVENT_TIMESTAMP  0x80
-#define EVENT_SIZE       0x100
-#define EVENT_CONNECTION 0x200
-#define EVENT_PERMISSION 0x400
-#define EVENT_UID        0x800
-#define EVENT_GID        0x1000
-#define EVENT_UNMONITOR  0x2000
-
+#define EVENT_NULL         0x0
+#define EVENT_CHECKSUM     0x1
+#define EVENT_RESOURCE     0x2
+#define EVENT_TIMEOUT      0x4
+#define EVENT_TIMESTAMP    0x8
+#define EVENT_SIZE         0x10
+#define EVENT_CONNECTION   0x20
+#define EVENT_PERMISSION   0x40
+#define EVENT_UID          0x80
+#define EVENT_GID          0x100
+#define EVENT_NONEXIST     0x200
+#define EVENT_INVALID      0x400
+#define EVENT_DATA         0x800
+#define EVENT_EXEC         0x1000
+#define EVENT_CHANGED      0x2000
+#define EVENT_ICMP         0x4000
 
 #define EVENT_DESCRIPTION(E) Event_get_description(E)
+#define EVENTACTION_DESCRIPTION(E) Event_get_action_description(E)
 #define IS_EVENT_SET(value, mask) ((value & mask) != 0)
 
+typedef struct myeventtable {
+  int id;
+  char description_failed[STRLEN];
+  char description_passed[STRLEN];
+} EventTable_T;
+
+extern EventTable_T Event_Table[];
+
 
 /**
  * This class implements the <b>event</b> processing machinery used by
  * monit. In monit an event is an object containing a Service_T
  * reference indicating the object where the event orginated, an id
- * specifying the event type and an optional message describing why
- * the event was fired.
+ * specifying the event type, a value representing up or down state
+ * and an optional message describing why the event was fired.
  *
  * Clients may use the function Event_post() to post events to the
  * event handler for processing.
  * 
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- * @version \$Id: event.h,v 1.13 2004/01/29 17:52:11 martinp Exp $
+ * @author Martin Pala, <martinp@tildeslash.com>
+ * @version \$Id: event.h,v 1.17 2004/07/31 20:15:07 martinp Exp $
  * @file
  */
 
-
-typedef struct Event_T *Event_T;
-
-
 /**
- * Construct and post a new Event
- * @param source The Service the event occured on
- * @param id The event type
+ * Post a new Event
+ * @param service The Service the event belongs to
+ * @param id The event identification
+ * @param state TRUE for failed, FALSE for passed event state
+ * @param action Description of the event action
  * @param s Optional message describing the event
  */
-void Event_post(Service_T source, long id, char *s, ...);
-
-
-/**
- * Destroy an Event object and release allocated resources. 
- * @param E An Event object reference
- */
-void Event_free(Event_T *E);
+void Event_post(Service_T service, long id, short state, EventAction_T action,
+  char *s, ...);
 
 
 /**
@@ -105,10 +106,10 @@ const char *Event_get_message(Event_T E);
 
 
 /**
- * Get a textual description of the event type. For instance if the
- * event type is EVENT_RESTART, the textual description is
- * "Restart". Likewise if the event type is EVENT_CHECKSUM the textual
- * description is "Checksum error" and so on.
+ * Get a textual description of actual event type. For instance if the
+ * event type is possitive EVENT_TIMESTAMP, the textual description is
+ * "Timestamp error". Likewise if the event type is negative EVENT_CHECKSUM
+ * the textual description is "Checksum recovery" and so on.
  * @param E An event object
  * @return A string describing the event type in clear text. If the
  * event type is not found NULL is returned.
@@ -116,4 +117,17 @@ const char *Event_get_message(Event_T E);
 const char *Event_get_description(Event_T E);
 
 
+/**
+ * Get a textual description of actual event action. For instance if the
+ * event type is possitive EVENT_NONEXIST, the textual description of
+ * failed state related action is "restart". Likewise if the event type is
+ * negative EVENT_CHECKSUM the textual description of recovery related action
+ * is "alert" and so on.
+ * @param E An event object
+ * @return A string describing the event type in clear text. If the
+ * event type is not found NULL is returned.
+ */
+const char *Event_get_action_description(Event_T E);
+
+
 #endif
