@@ -64,7 +64,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: files.c,v 1.49 2004/02/17 18:52:08 hauk Exp $
+ *  @version \$Id: files.c,v 1.50 2004/06/11 14:31:39 martinp Exp $
  *
  *  @file
  */
@@ -235,23 +235,6 @@ int create_pidfile(char *pidfile) {
 }
 
 
-/**
- * Secure check the monitrc file. The run control file must have the
- * same uid as the REAL uid of this process, it must have permissions
- * no greater than 700 and it must not be a symbolic link.  We check
- * these conditions here.
- * @param rcfile The monitrc file
- * @return TRUE if the test passed otherwise FALSE
- */
-int check_rcfile(char *rcfile) {
-
-  ASSERT(rcfile);
-  
-  return check_file_stat(rcfile, "control file", S_IRUSR | S_IWUSR | S_IXUSR);
-  
-}
-
-
 /**
  * Check if the file is a regular file
  * @param file A path to the file to check
