@@ -29,6 +29,7 @@
 
 #include "monitor.h"
 #include "protocol.h"
+#include "process.h"
 #include "ssl.h"
 #include "engine.h"
 
@@ -41,10 +42,12 @@ static void _gcppl(Port_T *);
 static void _gcdevice(Device_T *);
 static void _gcicmp(Icmp_T *);
 static void _gcpql(Resource_T *);
-static void _gcppil(ProcInfo_T *);
 static void _gcptl(Timestamp_T *);
 static void _gccmd(Command_T *);
-static void _gcpdil(DeviceInfo_T *);
+static void _gc_action(Action_T *);
+static void _gc_eventaction(EventAction_T *);
+static void _gc_event(Event_T *);
+static void _gc_inf(Info_T *);
 static void _gcpdl(Dependant_T *);
 static void _gcso(Size_T *);
 static void _gcchecksum(Checksum_T *);
@@ -61,7 +64,7 @@ static void _gcath(Auth_T *);
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: gc.c,v 1.48 2004/03/12 01:04:44 martinp Exp $
+ *  @version \$Id: gc.c,v 1.55 2004/08/16 20:00:30 martinp Exp $
  *
  *  @file
  */
@@ -76,6 +79,11 @@ void gc() {
 
   gc_protocols();
 
+  if(Run.doprocess) {
+    delprocesstree(&oldptree, oldptreesize);
+    delprocesstree(&ptree, ptreesize);
+  }
+  
   if(servicelist)
     _gc_service_list(&servicelist);
   
@@ -102,17 +110,17 @@ void gc() {
 void gc_mail_list(Mail_T *m) {
 
   ASSERT(m);
-  
+
   if((*m)->next)
     gc_mail_list(&(*m)->next);
-  
+
   FREE((*m)->to);
   FREE((*m)->from);
   FREE((*m)->subject);
   FREE((*m)->message);
   FREE((*m)->opt_message);
   FREE(*m);
-  
+
 }
 
 
@@ -123,9 +131,8 @@ static void _gc_service_list(Service_T *s) {
   
   ASSERT(s&&*s);
 
-  if((*s)->next) {
+  if((*s)->next)
     _gc_service_list(&(*s)->next);
-  }
   
   _gc_service(&(*s));
     
@@ -136,69 +143,68 @@ static void _gc_service(Service_T *s) {
 
   ASSERT(s&&*s);
   
-  if((*s)->portlist) {
+  if((*s)->portlist)
     _gcppl(&(*s)->portlist);
-  }
 
-  if((*s)->devicelist) {
+  if((*s)->devicelist)
     _gcdevice(&(*s)->devicelist);
-  }
 
-  if((*s)->icmplist) {
+  if((*s)->icmplist)
     _gcicmp(&(*s)->icmplist);
-  }
 
-  if((*s)->maillist) {
+  if((*s)->maillist)
     gc_mail_list(&(*s)->maillist);
-  }
 
-  if((*s)->resourcelist) {
+  if((*s)->resourcelist)
     _gcpql(&(*s)->resourcelist);
-  }
 
-  if((*s)->procinfo) {
-    _gcppil(&(*s)->procinfo);
-  }
+  if((*s)->inf)
+    _gc_inf(&(*s)->inf);
   
-  if((*s)->devinfo) {
-    _gcpdil(&(*s)->devinfo);
-  }
-  
-  if((*s)->timestamplist) {
+  if((*s)->timestamplist)
     _gcptl(&(*s)->timestamplist);
-  }
 
-  if((*s)->sizelist) {
+  if((*s)->sizelist)
     _gcso(&(*s)->sizelist);
-  }
 
-  if((*s)->checksum) {
+  if((*s)->checksum)
     _gcchecksum(&(*s)->checksum);
-  }
 
-  if((*s)->perm) {
+  if((*s)->perm)
     _gcperm(&(*s)->perm);
-  }
 
-  if((*s)->uid) {
+  if((*s)->uid)
     _gcuid(&(*s)->uid);
-  }
 
-  if((*s)->gid) {
+  if((*s)->gid)
     _gcgid(&(*s)->gid);
-  }
 
-  if((*s)->dependantlist) {
+  if((*s)->dependantlist)
     _gcpdl(&(*s)->dependantlist);
-  }
 
-  if((*s)->start) {
+  if((*s)->start)
     _gccmd(&(*s)->start);
-  }
-  
-  if((*s)->stop) {
+
+  if((*s)->stop)
     _gccmd(&(*s)->stop);
-  }
+
+  if((*s)->action_DATA)
+    _gc_eventaction(&(*s)->action_DATA);
+  
+  if((*s)->action_EXEC)
+    _gc_eventaction(&(*s)->action_EXEC);
+  
+  if((*s)->action_INVALID)
+    _gc_eventaction(&(*s)->action_INVALID);
+  
+  if((*s)->action_NONEXIST)
+    _gc_eventaction(&(*s)->action_NONEXIST);
+  
+  if((*s)->action_TIMEOUT)
+    _gc_eventaction(&(*s)->action_TIMEOUT);
+  
+  if((*s)->eventlist)
+    _gc_event(&(*s)->eventlist);
   
   FREE((*s)->name);
   FREE((*s)->path);
@@ -217,9 +223,8 @@ static void _gc_mail_server(MailServer_T *s) {
 
   if(!s&&!*s) return;
   
-  if((*s)->next) {
+  if((*s)->next)
     _gc_mail_server(&(*s)->next);
-  }
   
   FREE((*s)->host);
   FREE(*s);
@@ -240,21 +245,55 @@ static void _gccmd(Command_T *c) {
 }
 
 
+static void _gc_action(Action_T *a) {
+
+  ASSERT(a&&*a);
+  
+  if((*a)->exec)
+    _gccmd(&(*a)->exec);
+  FREE(*a);
+
+}
+
+
+static void _gc_eventaction(EventAction_T *e) {
+
+  ASSERT(e&&*e);
+  
+  _gc_action(&(*e)->failed);
+  _gc_action(&(*e)->passed);
+  FREE(*e);
+
+}
+
+
+static void _gc_event(Event_T *e) {
+
+  ASSERT(e&&*e);
+  
+  if((*e)->next)
+    _gc_event(&(*e)->next);
+
+  pthread_mutex_destroy(&(*e)->mutex);
+  (*e)->action= NULL;
+  FREE((*e)->message);
+  FREE(*e);
+
+}
+
+
 static void _gcppl(Port_T *p) {
   
   ASSERT(p&&*p);
 
-  if((*p)->next) {
+  if((*p)->next)
     _gcppl(&(*p)->next);
-  }
 
-  if((*p)->exec) {
-    _gccmd(&(*p)->exec);
-  }
+  if((*p)->action)
+    _gc_eventaction(&(*p)->action);
 
-  if((*p)->generic) {
+  if((*p)->generic)
     _gcgrc(&(*p)->generic);
-  }
 
   FREE((*p)->address);
   FREE((*p)->request);
@@ -271,13 +310,11 @@ static void _gcdevice(Device_T *d) {
   
   ASSERT(d&&*d);
 
-  if((*d)->next) {
+  if((*d)->next)
     _gcdevice(&(*d)->next);
-  }
 
-  if((*d)->exec) {
-    _gccmd(&(*d)->exec);
-  }
+  if((*d)->action)
+    _gc_eventaction(&(*d)->action);
 
   FREE(*d);
 
@@ -288,13 +325,11 @@ static void _gcicmp(Icmp_T *i) {
   
   ASSERT(i&&*i);
 
-  if((*i)->next) {
+  if((*i)->next)
     _gcicmp(&(*i)->next);
-  }
 
-  if((*i)->exec) {
-    _gccmd(&(*i)->exec);
-  }
+  if((*i)->action)
+    _gc_eventaction(&(*i)->action);
 
   FREE(*i);
 
@@ -305,33 +340,25 @@ static void _gcpql(Resource_T *q) {
 
   ASSERT(q);
 
-  if((*q)->next) {
+  if((*q)->next)
     _gcpql(&(*q)->next);
-  }
   
-  if((*q)->exec) {
-    _gccmd(&(*q)->exec);
-  }
+  if((*q)->action)
+    _gc_eventaction(&(*q)->action);
 
   FREE(*q);
   
 }
 
 
-static void _gcppil(ProcInfo_T *pi) {
+static void _gc_inf(Info_T *i) {
 
-  ASSERT(pi);
+  ASSERT(i);
 
-  FREE(*pi);
+  if((*i)->cs_sum)
+    FREE((*i)->cs_sum);
 
-}
-
-
-static void _gcpdil(DeviceInfo_T *di) {
-
-  ASSERT(di);
-
-  FREE(*di);
+  FREE(*i);
 
 }
 
@@ -340,13 +367,12 @@ static void _gcptl(Timestamp_T *p) {
   
   ASSERT(p);
 
-  if((*p)->next) {
+  if((*p)->next)
     _gcptl(&(*p)->next);
-  }
 
-  if((*p)->exec) {
-    _gccmd(&(*p)->exec);
-  }
+  if((*p)->action)
+    _gc_eventaction(&(*p)->action);
+
   FREE(*p);
 
 }
@@ -359,8 +385,8 @@ static void _gcso(Size_T *s) {
   if((*s)->next)
     _gcso(&(*s)->next);
 
-  if((*s)->exec)
-    _gccmd(&(*s)->exec);
+  if((*s)->action)
+    _gc_eventaction(&(*s)->action);
 
   FREE(*s);
 
@@ -371,8 +397,8 @@ static void _gcchecksum(Checksum_T *s) {
   
   ASSERT(s);
 
-  if((*s)->exec)
-    _gccmd(&(*s)->exec);
+  if((*s)->action)
+    _gc_eventaction(&(*s)->action);
 
   FREE((*s)->hash);
   FREE(*s);
@@ -384,8 +410,8 @@ static void _gcperm(Perm_T *s) {
   
   ASSERT(s);
 
-  if((*s)->exec)
-    _gccmd(&(*s)->exec);
+  if((*s)->action)
+    _gc_eventaction(&(*s)->action);
 
   FREE(*s);
 
@@ -396,8 +422,8 @@ static void _gcuid(Uid_T *s) {
   
   ASSERT(s);
 
-  if((*s)->exec)
-    _gccmd(&(*s)->exec);
+  if((*s)->action)
+    _gc_eventaction(&(*s)->action);
 
   FREE(*s);
 
@@ -408,8 +434,8 @@ static void _gcgid(Gid_T *s) {
   
   ASSERT(s);
 
-  if((*s)->exec)
-    _gccmd(&(*s)->exec);
+  if((*s)->action)
+    _gc_eventaction(&(*s)->action);
 
   FREE(*s);
 
@@ -420,9 +446,8 @@ static void _gcpdl(Dependant_T *d) {
 
   ASSERT(d);
 
-  if((*d)->next) {
+  if((*d)->next)
     _gcpdl(&(*d)->next);
-  }
 
   FREE((*d)->dependant);
   FREE(*d);
@@ -434,15 +459,13 @@ static void _gcgrc(Generic_T *g) {
 
   ASSERT(g);
 
-  if((*g)->next) {
+  if((*g)->next)
     _gcgrc(&(*g)->next);
-  }
 
   FREE((*g)->send);
 #ifdef HAVE_REGEX_H
-  if ((*g)->expect!=NULL) {
+  if ((*g)->expect!=NULL)
     regfree((*g)->expect);
-  }
 #endif
   FREE((*g)->expect);
   FREE(*g);
@@ -454,12 +477,12 @@ static void _gcath(Auth_T *c) {
 
   ASSERT(c);
 
-  if((*c)->next) {
+  if((*c)->next)
     _gcath(&(*c)->next);
-  }
 
   FREE((*c)->uname);
   FREE((*c)->passwd);
   FREE(*c);
   
 }
+
