@@ -45,7 +45,7 @@ static unsigned char decode(char c);
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: base64.c,v 1.13 2004/01/29 17:52:11 martinp Exp $
+ *  @version \$Id: base64.c,v 1.14 2004/04/04 08:08:11 hauk Exp $
  *
  *  @file
  */
@@ -128,7 +128,7 @@ char *encode_base64(int size, unsigned char *src) {
  */
 int decode_base64(unsigned char *dest, const char *src) {
 
-  if(src ) {
+  if(src && *src) {
   
     unsigned char *p= dest;
     int k, l= strlen(src)+1;
