@@ -103,7 +103,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala <martinp@tildeslash.com>
  *
- *  @version \$Id: engine.c,v 1.56 2004/03/13 00:10:18 martinp Exp $
+ *  @version \$Id: engine.c,v 1.57 2004/06/13 21:04:26 martinp Exp $
  *
  *  @file 
  */
@@ -141,7 +141,7 @@ static Socket_T socket_producer(int, int, void*);
  */
 void start_httpd(int port, int backlog, char *bindAddr) {
 
-  Socket_T S;
+  Socket_T S= NULL;
 
   stopped= Run.stopped;
 
@@ -192,10 +192,6 @@ void start_httpd(int port, int backlog, char *bindAddr) {
       
     }
 
-    if(S) {
-      socket_free(&S);
-    }
-    
     delete_ssl_server_socket(mySSLServerConnection);  
     close_socket(myServerSocket);
 
