@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -33,12 +33,11 @@
  */
 #define DEFAULT_QUEUE_LEN  10
 
-struct host_allow {
+typedef struct host_allow {
   char *name;  
   /* For internal use */
   struct host_allow *next;
-};
-typedef struct host_allow* HostsAllow;
+} *HostsAllow;
 
 /* Public prototypes */
 void start_httpd(int port, int backlog, char *bindAddr);
