@@ -101,7 +101,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: processor.c,v 1.40 2004/04/04 08:08:11 hauk Exp $
+ *  @version \$Id: processor.c,v 1.45 2004/08/02 20:06:39 martinp Exp $
  *
  *  @file
  */
@@ -138,17 +138,17 @@ static int get_next_token(char *s, int *cursor, char **r);
  * function. 
  * @param s A Socket_T representing the client connection
  */
-void *http_processor(void *s) {
-  Socket_T S= s;
-  
-  if(! can_read(socket_get_socket(S), REQUEST_TIMEOUT)) {
-    internal_error(S, SC_REQUEST_TIMEOUT, "Time out when handling the Request");
-    goto shutdown;
+void *http_processor(Socket_T s) {
+
+  if(! can_read(socket_get_socket(s), REQUEST_TIMEOUT)) {
+    internal_error(s, SC_REQUEST_TIMEOUT, "Time out when handling the Request");
+  } else {
+    do_service(s);
   }
-  do_service(S);
- shutdown:
-  socket_free(&S);
+  socket_free(&s);
+
   return NULL;
+
 }
 
 
@@ -508,6 +508,7 @@ static void send_response(HttpResponse res) {
     char server[STRLEN];
     char *headers= get_headers(res);
 
+    res->is_committed= TRUE;
     get_date(date, STRLEN);
     get_server(server, STRLEN);
     socket_print(S, "%s %d %s\r\n", res->protocol, res->status,
@@ -580,7 +581,7 @@ static HttpResponse create_HttpResponse(Socket_T S) {
   res->S= S;
   res->bufsize= 0;
   res->bufused= 0;
-  res->status= 200;
+  res->status= SC_OK;
   res->outputbuffer= NULL;
   res->is_committed= FALSE;
   res->protocol= SERVER_PROTOCOL;
@@ -595,7 +596,6 @@ static HttpResponse create_HttpResponse(Socket_T S) {
 static void create_headers(HttpRequest req) {
   Socket_T S;
   char *value;
-  HttpHeader p;
   HttpHeader header;
   char line[REQ_STRLEN];
 
@@ -605,22 +605,16 @@ static void create_headers(HttpRequest req) {
 	break;
     if(IS(line, "\r\n") || IS(line, "\n"))
 	break;
-    if( NULL != (value= strchr(line, ':'))) {
+    if(NULL != (value= strchr(line, ':'))) {
       NEW(header);
-      *value++= '\0';
+      *value++= 0;
       trim(line);
       trim(value);
-      /* ATTENTION: Please double check this calculations: */
       chomp(value, (REQ_STRLEN-(value-line)));
       header->name= xstrdup(line);
       header->value= xstrdup(value);
-      if(req->headers) {
-	for(p= req->headers; p->next; p= p->next)
-	  /* Empty */;
-	p->next= header;
-      } else {
-	req->headers= header;
-      }
+      header->next= req->headers;
+      req->headers= header;
     }
   }
 }
@@ -724,7 +718,7 @@ static void destroy_HttpResponse(HttpResponse res) {
 
 
 /**
- * Free a (linked list of) http entry ojbect(s). Both HttpHeader and
+ * Free a (linked list of) http entry object(s). Both HttpHeader and
  * HttpParameter are of this type.
  */
 static void destroy_entry(void *p) {
@@ -788,13 +782,13 @@ static int basic_authenticate(HttpRequest req) {
   if(*uname==0 || *password==0) {
     return FALSE;
   }
-  /* Check if user 'uname' exist */
+  /* Check if user exist */
   if(NULL==get_user_credentials(uname)) {
     log("Warning: Client '%s' supplied unknown user '%s'"
 	" accessing monit httpd\n", socket_get_remote_host(req->S), uname); 
     return FALSE;
   }
-  /* Check if 'uname' has supplied the right passwd */
+  /* Check if user has supplied the right password */
   if(! compare_user_credentials(uname,  password)) {
     log("Warning: Client '%s' supplied wrong password for user '%s'"
 	" accessing monit httpd\n", socket_get_remote_host(req->S), uname); 
