@@ -125,7 +125,7 @@ typedef struct request {
 typedef struct response {
   int status;
   Socket_T S;
-  char *protocol;
+  const char *protocol;
   size_t bufsize;
   size_t bufused;
   int is_committed;
@@ -150,7 +150,7 @@ struct  ServiceImpl {
 struct ServiceImpl Impl;
 
 /* Public prototypes */
-void *http_processor(void *);
+void *http_processor(Socket_T);
 char *get_headers(HttpResponse res);
 void set_status(HttpResponse res, int status);
 const char *get_status_string(int status_code);
