@@ -43,6 +43,10 @@
 #include <stdarg.h>
 #endif
 
+#ifdef HAVE_STDIO_H
+#include <stdio.h>
+#endif
+
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -80,6 +84,10 @@
 #define TRUE               1
 #define FALSE              0
 
+#define MONITOR_NOT        0
+#define MONITOR_YES        1
+#define MONITOR_INIT       2
+
 #define MODE_ACTIVE        0
 #define MODE_PASSIVE       1
 #define MODE_MANUAL        2
@@ -94,17 +102,19 @@
 #define TIME_HOUR          3600
 #define TIME_DAY           86400
 
+#define ACTION_IGNORE      0
 #define ACTION_ALERT       1
 #define ACTION_RESTART     2
 #define ACTION_STOP        3
 #define ACTION_EXEC        4
 #define ACTION_UNMONITOR   5
+#define ACTION_START       6
 
 #define TYPE_DEVICE        0
 #define TYPE_DIRECTORY     1
 #define TYPE_FILE          2
 #define TYPE_PROCESS       3
-#define TYPE_REMOTE        4
+#define TYPE_HOST          4
 
 #define RESOURCE_ID_CPU_PERCENT       1
 #define RESOURCE_ID_MEM_PERCENT       2
@@ -127,6 +137,7 @@
 #define UNIT_MEGABYTE      1048580
 #define UNIT_GIGABYTE      1073740000
 
+#define HASH_UNKNOWN       0
 #define HASH_MD5           1
 #define HASH_SHA1          2
 #define DEFAULT_HASH       HASH_MD5
@@ -182,6 +193,38 @@ Sigfunc *signal(int signo, Sigfunc * func);
 
 /* --------------------------------------------------------- Data structures */
 
+
+/**
+ * Defines a Command with ARGMAX optional arguments. The arguments
+ * array must be NULL terminated and the first entry is the program
+ * itself. In addition, a user and group may be set for the Command
+ * which means that the Command should run as a certain user and with
+ * certain group.
+ */
+typedef struct mycommand {
+  char *arg[ARGMAX];                             /**< Program with arguments */
+  int   length;                       /**< The length of the arguments array */
+  int   has_uid;          /**< TRUE if a new uid is defined for this Command */
+  uid_t uid;         /**< The user id to switch to when running this Command */
+  int   has_gid;          /**< TRUE if a new gid is defined for this Command */
+  gid_t gid;        /**< The group id to switch to when running this Command */
+} *Command_T;
+
+
+/** Defines an event action object */
+typedef struct myaction {
+    int       id;                                     /**< Action to be done */
+    Command_T exec;                    /**< Optional command to be executed  */
+} *Action_T;
+
+
+/** Defines event's up and down actions */
+typedef struct myeventaction {
+  Action_T  failed;                  /**< Action in the case of failure down */
+  Action_T  passed;                    /**< Action in the case of failure up */
+} *EventAction_T;
+
+
 /** Defines a mailinglist object */
 typedef struct mymail {
   char *to;                         /**< Mail address for alert notification */
@@ -199,6 +242,7 @@ typedef struct mymail {
 /** Defines a mail server address */
 typedef struct mymailserver {
   char *host;     /**< Server host address, may be a IP or a hostname string */
+  int   port;                                               /**< Server port */
   /** For internal use */
   struct mymailserver *next;        /**< Next server to try on connect error */
 } *MailServer_T;
@@ -213,22 +257,44 @@ typedef struct myauthentication {
 } *Auth_T;
 
 
-/** Defines process tree */
+/** Defines process tree - data storage backend*/
 typedef struct myprocesstree {
   int  pid;
   int  ppid;
+  int  status_flag;
+  
   int  visited;
-  long mem_kbyte;
   int  children_num;
   int  children_sum;
-  long mem_kbyte_sum;
+  int  cpu_percent;
+  int  cpu_percent_sum;
+  unsigned long mem_kbyte;
+  unsigned long mem_kbyte_sum;
 
   /** For internal use */
+  double time;                                             /**< 1/10 seconds */
+  double time_prev;                                        /**< 1/10 seconds */
+  long  cputime;                                           /**< 1/10 seconds */
+  long  cputime_prev;                                      /**< 1/10 seconds */
+  
   struct myprocesstree  *parent;
   struct myprocesstree **children;
 } ProcessTree_T;
 
 
+/** Defines data for systemwide statistic */
+typedef struct mysysteminfo {
+  time_t collected;                            /**< When were data collected */
+  double loadavg[3];                                /**< Load average triple */
+  unsigned long mem_kbyte_max;               /**< Maximal system real memory */
+  unsigned long total_mem_kbyte; /**< Total real memory in use in the system */
+  int    total_mem_percent;      /**< Total real memory in use in the system */
+  int    total_cpu_user_percent;   /**< Total CPU in use in user space (pct.)*/
+  int    total_cpu_syst_percent; /**< Total CPU in use in kernel space (pct.)*/
+  int    total_cpu_wait_percent;      /**< Total CPU in use in waiting (pct.)*/
+} SystemInfo_T;
+
+
 /** Defines data for application runtime */
 struct myrun {
   int  stopped;         /**< TRUE if monit was stopped. Flag used by threads */
@@ -262,8 +328,6 @@ struct myrun {
   int  testing;   /**< Running in configuration testing mode - TRUE or FALSE */
   int  wait_start;         /**< Number of threads waiting for process' start */
 
-  double loadavg[3];                                /**< Load average triple */
-
        /** An object holding program relevant "environment" data, see; env.c */
   struct myenvironment {
     char *user;             /**< The the effective user running this program */
@@ -287,23 +351,6 @@ struct myrun {
 };
 
 
-/**
- * Defines a Command with ARGMAX optional arguments. The arguments
- * array must be NULL terminated and the first entry is the program
- * itself. In addition, a user and group may be set for the Command
- * which means that the Command should run as a certain user and with
- * certain group.
- */
-typedef struct mycommand {
-  char *arg[ARGMAX];                             /**< Program with arguments */
-  int length;                         /**< The length of the arguments array */
-  int has_uid;            /**< TRUE if a new uid is defined for this Command */
-  uid_t uid;         /**< The user id to switch to when running this Command */
-  int has_gid;            /**< TRUE if a new gid is defined for this Command */
-  gid_t gid;        /**< The group id to switch to when running this Command */
-} *Command_T;
-
-
 /** Defines a protocol object with protocol functions */
 typedef struct myprotocol {
   const char *name;                                       /**< Protocol name */
@@ -337,12 +384,10 @@ typedef struct myport {
   char *pathname;                   /**< Pathname, in case of an UNIX socket */
   char *address;               /**< Human readable destination of the socket */
   Generic_T generic;                                /**< Generic test handle */
-  int  action;                                /**< Action in case of failure */
-  int  event_flag;              /**< TRUE if an event occured on this object */
-  int  event_handled;        /**< TRUE if this event has been handled before */
-  Command_T exec;      /**< Optional command to be executed upon a port event*/
   int timeout;   /**< The timeout in seconds to wait for connect or read i/o */
   int is_available;                /**< TRUE if the server/port is available */
+  double response;                      /**< Socket connection response time */
+  EventAction_T action;  /**< Description of the action upon event occurence */
   
   struct {
     int use_ssl;          /**< TRUE if this port requires SSL for connection */
@@ -362,11 +407,9 @@ typedef struct myport {
 typedef struct myicmp {
   int type;                                              /**< ICMP type used */
   int timeout;              /**< The timeout in seconds to wait for response */
-  int action;                                 /**< Action in case of failure */
-  int event_flag;               /**< TRUE if an event occured on this object */
-  int  event_handled;        /**< TRUE if this event has been handled before */
-  Command_T exec;     /**< Optional command to be executed upon a icmp event */
   int is_available;                     /**< TRUE if the server is available */
+  double response;                              /**< ICMP ECHO response time */
+  EventAction_T action;  /**< Description of the action upon event occurence */
   
   /** For internal use */
   struct myicmp *next;                               /**< next icmp in chain */
@@ -388,10 +431,7 @@ typedef struct myresource {
   int  operator;                                    /**< Comparison operator */
   int  cycle;                                     /**< Cycle overrun counter */
   int  max_cycle;                                   /**< Cycle overrun limit */
-  int  action;                                /**< Action in case of failure */
-  int  event_flag;              /**< TRUE if an event occured on this object */
-  int  event_handled;        /**< TRUE if this event has been handled before */
-  Command_T exec;   /**< Optional command to be executed upon a resurce event*/
+  EventAction_T action;  /**< Description of the action upon event occurence */
   
   /** For internal use */
   struct myresource *next;                       /**< next resource in chain */
@@ -402,12 +442,9 @@ typedef struct myresource {
 typedef struct mytimestamp {
   int  operator;                                    /**< Comparison operator */
   int  time;                                        /**< Timestamp watermark */
-  int  action;                                /**< Action in case of failure */
   int  test_changes;            /**< TRUE if we only should test for changes */
   time_t timestamp; /**< The original last modified timestamp for this object*/
-  int  event_flag;              /**< TRUE if an event occured on this object */
-  int  event_handled;        /**< TRUE if this event has been handled before */
-  Command_T exec; /**< Optional command to be executed upon a timestamp event*/
+  EventAction_T action;  /**< Description of the action upon event occurence */
   
   /** For internal use */
   struct mytimestamp *next;                     /**< next timestamp in chain */
@@ -418,12 +455,8 @@ typedef struct mytimestamp {
 typedef struct mysize {
   int  operator;                                    /**< Comparison operator */
   unsigned long size;                                    /**< Size watermark */
-  int  action;                                /**< Action in case of failure */
   int  test_changes;            /**< TRUE if we only should test for changes */
-  unsigned long runsize;              /**< The original size for this object */
-  int  event_flag;              /**< TRUE if an event occured on this object */
-  int  event_handled;        /**< TRUE if this event has been handled before */
-  Command_T exec;     /**< Optional command to be executed upon a size event */
+  EventAction_T action;  /**< Description of the action upon event occurence */
   
   /** For internal use */
   struct mysize *next;                          /**< next timestamp in chain */
@@ -435,92 +468,83 @@ typedef struct mychecksum {
   char     *hash;                 /**< A checksum hash computed for the path */
   int       type;                   /**< The type of hash (e.g. md5 or sha1) */
   int       length;                                  /**< Length of the hash */
-  int       action;                           /**< Action in case of failure */
-  int       event_flag;         /**< TRUE if an event occured on this object */
-  Command_T exec; /**< Optional command to be executed upon a checksum event */
+  int       test_changes;       /**< TRUE if we only should test for changes */
+  EventAction_T action;  /**< Description of the action upon event occurence */
 } *Checksum_T;
 
 
 /** Defines permission object */
 typedef struct myperm {
   int       perm;                                     /**< Access permission */
-  int       action;                           /**< Action in case of failure */
-  int       event_flag;         /**< TRUE if an event occured on this object */
-  Command_T exec;     /**< Optional command to be executed upon a perm event */
-  int       has_error;       /**< TRUE if the service has a permission error */
+  EventAction_T action;  /**< Description of the action upon event occurence */
 } *Perm_T;
 
 
 /** Defines uid object */
 typedef struct myuid {
   uid_t     uid;                                            /**< Owner's uid */
-  int       action;                           /**< Action in case of failure */
-  int       event_flag;         /**< TRUE if an event occured on this object */
-  Command_T exec;      /**< Optional command to be executed upon a uid event */
-  int       has_error;              /**< TRUE if the service has a UID error */
+  EventAction_T action;  /**< Description of the action upon event occurence */
 } *Uid_T;
 
 
 /** Defines gid object */
 typedef struct mygid {
   gid_t     gid;                                            /**< Owner's gid */
-  int       action;                           /**< Action in case of failure */
-  int       event_flag;         /**< TRUE if an event occured on this object */
-  Command_T exec;       /**< Optional command to be executed upon a gid event*/
-  int       has_error;              /**< TRUE if the service has a GID error */
+  EventAction_T action;  /**< Description of the action upon event occurence */
 } *Gid_T;
 
 
-/** Defines procfs (or other mechanism) data */
-typedef struct myprocinfo {
-  int  pid;
-  int  ppid;
-  int  status_flag;
-  long mem_kbyte;    
-  int  mem_percent;                                      /**< pecentage * 10 */
-  int  cpu_percent;                                      /**< pecentage * 10 */
-  int  children;
-  long total_mem_kbyte;
-  int  total_mem_percent;                                /**< pecentage * 10 */
-
-  /* private for calculating cpu_percent */
-  double time_prev;                                        /**< 1/10 seconds */
-  long  cputime;                                           /**< 1/10 seconds */
-  long  cputime_prev;                                      /**< 1/10 seconds */
-} *ProcInfo_T;
-
-
-/** Defines filesystem device data */
-typedef struct mydeviceinfo {
-  long f_bsize;                                     /**< Transfer block size */
-  long f_blocks;                        /**< Total data blocks in filesystem */
-  long f_blocksfree;             /**< Free blocks available to non-superuser */
-  long f_blocksfreetotal;                     /**< Free blocks in filesystem */
-  long f_files;                          /**< Total file nodes in filesystem */
-  long f_filesfree;                       /**< Free file nodes in filesystem */
-
-  int  event_handled;        /**< TRUE if this event has been handled before */
-  /** For internal use */
-  char mntpath[STRLEN];        /**< Filesystem file, directory or mountpoint */
-} *DeviceInfo_T;
-
-
 /** Defines filesystem device configuration */
 typedef struct mydevice {
   int  resource;                        /**< Whether to check inode or space */
   int  operator;                                    /**< Comparison operator */
   long limit_absolute;                               /**< Watermark - blocks */
   int  limit_percent;                               /**< Watermark - percent */
-  int  action;                                /**< Action in case of failure */
-  int  event_flag;              /**< TRUE if an event occured on this object */
-  int  event_handled;        /**< TRUE if this event has been handled before */
-  Command_T exec;   /**< Optional command to be executed upon a device event */
+  EventAction_T action;  /**< Description of the action upon event occurence */
 
   /** For internal use */
   struct mydevice *next;                           /**< next device in chain */
 } *Device_T;
 
 
+/** Defines service data */
+typedef struct myinfo {
+
+  /* Shared */
+  mode_t  st_mode;                                           /**< Permission */
+  uid_t   st_uid;                                           /**< Owner's uid */
+  gid_t   st_gid;                                           /**< Owner's gid */
+  time_t  timestamp;                                          /**< Timestamp */
+
+  /* Device specific */
+  long   f_bsize;                                   /**< Transfer block size */
+  long   f_blocks;                      /**< Total data blocks in filesystem */
+  long   f_blocksfree;           /**< Free blocks available to non-superuser */
+  long   f_blocksfreetotal;                   /**< Free blocks in filesystem */
+  long   f_files;                        /**< Total file nodes in filesystem */
+  long   f_filesfree;                     /**< Free file nodes in filesystem */
+  char   mntpath[STRLEN];      /**< Filesystem file, directory or mountpoint */
+
+  /* FIle specific */
+  size_t st_size;                                                 /**< Size */
+  char  *cs_sum;                                              /**< Checksum */
+
+  /* Process specific */
+  int    pid;
+  int    ppid;
+  int    status_flag;
+  int    children;
+  long   mem_kbyte;    
+  long   total_mem_kbyte;
+  int    mem_percent;                                    /**< pecentage * 10 */
+  int    total_mem_percent;                              /**< pecentage * 10 */
+  int    cpu_percent;                                    /**< pecentage * 10 */
+  int    total_cpu_percent;                              /**< pecentage * 10 */
+  time_t uptime;                                         /**< Process uptime */
+
+} *Info_T;
+
+
 /** Defines service data */
 typedef struct myservice {
 
@@ -528,10 +552,8 @@ typedef struct myservice {
   char *name;                                  /**< Service descriptive name */
   char *group;                                       /**< Service group name */
   int (*check)(struct myservice *);       /**< Service verification function */
-  Command_T start;                    /**< The start command for the service */
-  Command_T stop;                      /**< The stop command for the service */
   int  type;                                     /**< Monitored service type */
-  int  do_monitor; /**< Monitor flag, if FALSE, the service is not monitored */
+  int  monitor;                                      /**< Monitor state flag */
   int  mode;                            /**< Monitoring mode for the service */
   int  ncycle;                          /**< The number of the current cycle */
   int  nstart;           /**< The number of current starts with this service */
@@ -539,17 +561,17 @@ typedef struct myservice {
   int  to_cycle;                                  /**< Timeout cycle ceiling */
   int  every;                        /**< Check this program at given cycles */
   int  nevery;          /**< Counter for every.  When nevery == every, check */
-  int  def_every;              /**< TRUE if every is defined for the service */
   int  def_timeout;          /**< TRUE if timeout is defined for the service */
-  int  def_procinfo;        /**< TRUE if procinfo is defined for the service */
+  int  def_every;              /**< TRUE if every is defined for the service */
   int  visited;      /**< Service visited flag, set if dependencies are used */
   int  depend_visited;/**< Depend visited flag, set if dependencies are used */
-  int  event_handled; /**< TRUE if an event has been handled on this service */
+  Command_T start;                    /**< The start command for the service */
+  Command_T stop;                      /**< The stop command for the service */
 
+  Dependant_T dependantlist;                     /**< Dependant service list */
   Mail_T      maillist;                  /**< Alert notification mailinglist */
-  Dependant_T dependantlist;                      /**<Dependant service list */
 
-  /** Tests */
+  /** Test rules and event handlers */
   Checksum_T  checksum;                                  /**< Checksum check */
   Device_T    devicelist;                             /**< Device check list */
   Gid_T       gid;                                            /**< Gid check */
@@ -561,9 +583,31 @@ typedef struct myservice {
   Timestamp_T timestamplist;                       /**< Timestamp check list */
   Uid_T       uid;                                            /**< Uid check */
   
+  /** General event handlers */
+  EventAction_T action_DATA;       /**< Description of the action upon event */
+  EventAction_T action_EXEC;       /**< Description of the action upon event */
+  EventAction_T action_INVALID;    /**< Description of the action upon event */
+  EventAction_T action_NONEXIST;   /**< Description of the action upon event */
+  EventAction_T action_TIMEOUT;    /**< Description of the action upon event */
+
   /** Runtime parameters */
-  DeviceInfo_T devinfo;                       /**< Data for the device check */
-  ProcInfo_T   procinfo;                      /**< Data for the procfs check */
+  unsigned long long error;                          /**< Error flags bitmap */
+  Info_T             inf;                          /**< Service check result */
+  time_t             collected;                /**< When were data collected */
+
+
+  /** Events */
+  struct myevent {
+    int               id;                      /**< The event identification */
+    struct myservice *source;                              /**< Event source */
+    short             state;            /**< TRUE if failed, FALSE if passed */
+    short             state_changed;              /**< TRUE if state changed */
+    char             *message;    /**< Optional message describing the event */
+    EventAction_T     action;           /**< Description of the event action */
+    /** For internal use */
+    pthread_mutex_t   mutex;      /**< Mutex used for action synchronization */
+    struct myevent   *next;                         /**< next event in chain */
+  } *eventlist;                                     /**< Pending events list */
 
   /** Context specific parameters */
   char *path;   /**< Path to the device, file, directory or process pid file */
@@ -576,6 +620,9 @@ typedef struct myservice {
 } *Service_T;
 
 
+typedef struct myevent *Event_T;
+
+
 /* -------------------------------------------------------- Global variables */
 
 
@@ -583,9 +630,12 @@ char   *prog;                                  /**< The Name of this Program */
 struct myrun Run;                      /**< Struct holding runtime constants */
 Service_T servicelist;                /**< The service list (created in p.y) */
 Service_T servicelist_conf;   /**< The service list in conf file (c. in p.y) */
-ProcessTree_T *ptree;                        /**< Global process information */
-int            ptreesize;                    /**< Global process information */
+SystemInfo_T systeminfo;                              /**< System infomation */
 
+extern ProcessTree_T *ptree;     
+extern int            ptreesize;    
+extern ProcessTree_T *oldptree;  
+extern int            oldptreesize; 
 
 /* ------------------------------------------------------- Public prototypes */
 
@@ -613,27 +663,23 @@ int   isreg_file(char *);
 char *stripfilename(char*);
 int   exist_file(char *);
 char *get_RFC822date(long *);
-char *get_uptime(time_t delta, char *);
-char *get_process_uptime(char *, char *);
+char *get_uptime(time_t, char *);
+time_t get_process_uptime(char *);
 char *get_checksum(char *, int);
-int   check_hash(char *, char *, int);
-char *url_encode(char *uri);
-char *url_decode(char *url);
+char *url_encode(char *);
+char *url_decode(char *);
 char *get_basic_authentication_header();
 Auth_T get_user_credentials(char *);
 int   compare_user_credentials(char *, char *);
 int   parse(char *);
-void  control(const char *);
-void  control_group(const char *, const char *);
-void  control_process(const char *, const char *);
-void  d_check_service(const char *, const char *);
-void  check_service(const char *, const char *);
+void  control_service(const char *, const char *);
+void  control_service_daemon(const char *, const char *);
 void  setup_dependants();
 void  reset_depend();
 void  spawn(Service_T, Command_T, const char *);
 void  status();
 int   log_init() ;
-void  log_log(const char *format, ...) ;
+void  log_log(const char *, ...) ;
 void  log_close();
 void  validate() ;
 void  daemonize() ;
@@ -644,22 +690,19 @@ time_t get_timestamp(char *, mode_t);
 void  finalize_files();
 char *find_rcfile();    
 int   create_pidfile(char *);
-int   check_rcfile(char *);
 int   kill_daemon(int);
 int   exist_daemon(); 
 void  sendmail(Mail_T);
-int   sock_msg(int sock, char *, ...);
+int   sock_msg(int, char *, ...);
 void  init_env();
-void  destroy_env();
 void *xmalloc (int);
 void *xcalloc(long, long);
 char *xstrdup(const char *);
 char *xstrndup(const char *, long);
 void *xresize(void *, long);
 void  monit_http(int);
-int   check_httpd();
 int   can_http();
-char *format(const char *s, va_list ap, long *);
+char *format(const char *, va_list, long *);
 void  redirect_stdfd();
 void  fd_close();
 pid_t getpgid(pid_t);
@@ -677,6 +720,8 @@ int  check_device(Service_T);
 int  check_file(Service_T);
 int  check_directory(Service_T);
 int  check_remote_host(Service_T);
-
+int  sha_md5_stream (FILE *, void *, void *);
+void reset_procinfo(Service_T);
+void printhash(char *);  
 
 #endif
