@@ -43,6 +43,9 @@
 #define ICMP_SIZE sizeof(struct icmphdr)
 #else
 #define ICMP_SIZE sizeof(struct icmp)
+#endif
+
+#ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
 #endif
 
@@ -131,7 +134,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.c,v 1.51 2004/02/16 22:42:32 hauk Exp $
+ *  @version \$Id: net.c,v 1.57 2004/08/29 19:26:48 martinp Exp $
  *
  *  @file
  */
@@ -147,35 +150,6 @@ static unsigned short checksum_ip(unsigned char *, int);
 /* ------------------------------------------------------------------ Public */
 
 
-/**
- * Test INET Socket connection to host on port with the specified
- * socket protocol type. The protocol should be either SOCK_STREAM or
- * SOCK_DGRAM.
- * @param hostname The host to connect to
- * @param port The portnumber to connect to
- * @param protocol The socket protocol to use
- * @return TRUE if the connect succeeded otherwise FALSE.
- */
-int check_connect(const char *hostname, int port, int protocol) {
-
-  int socket;
-  int rv= TRUE;
-
-  ASSERT(hostname);
-  
-  if((socket= create_socket(hostname, port, protocol, NET_TIMEOUT)) < 0) {
-    rv= FALSE;
-  } else if(! check_socket(socket)) {
-    rv= FALSE;
-  }
-  
-  close_socket(socket);
-  
-  return rv;
-  
-}
-
-
 /**
  * Check if the hostname resolves
  * @param hostname The host to check
@@ -603,9 +577,9 @@ int sock_read(int socket, void *buffer, int size, int timeout) {
  * Create a ICMP socket against hostname, send echo and wait for response.
  * @param hostname The host to open a socket at
  * @param timeout If response will not come within timeout seconds abort
- * @return TRUE on succes, FALSE on error
+ * @return response time on succes, -1 on error
  */
-int icmp_echo(const char *hostname, int timeout) {
+double icmp_echo(const char *hostname, int timeout) {
 
   struct hostent *hp;
   struct sockaddr_in sin;
@@ -619,23 +593,25 @@ int icmp_echo(const char *hostname, int timeout) {
   struct icmp *icmphdrin= NULL;
   struct icmp *icmphdrout= NULL;
 #endif
-  struct timeval tv;
   size_t size;
   fd_set rset;
   int s;
   int n= 0;
-  int rv= FALSE;
   int sol_ip;
   unsigned ttl= 255;
   char buf[STRLEN];
+  struct timeval tv;
+  struct timeval t1;
+  struct timeval t2;
+  double response= -1;
   
   ASSERT(hostname);
 
   if((hp= gethostbyname(hostname)) == NULL)
-    return FALSE;
+    return response;
 
   if((s= socket(AF_INET, SOCK_RAW, IPPROTO_ICMP)) < 0)
-    return FALSE;
+    return response;
 
 #ifdef HAVE_SOL_IP
   sol_ip = SOL_IP;
@@ -650,34 +626,35 @@ int icmp_echo(const char *hostname, int timeout) {
   if(setsockopt(s, sol_ip, IP_TTL, (char *)&ttl, sizeof(ttl)) < 0)
     goto error;
 
+  tv.tv_sec= timeout;
+  tv.tv_usec= 0;
+
   NEW(icmphdrout);
 #ifdef HAVE_SOL_IP
   icmphdrout->code= 0;
   icmphdrout->type= ICMP_ECHO;
   icmphdrout->un.echo.id= getpid();
   icmphdrout->un.echo.sequence= 0;
-  icmphdrout->checksum= checksum_ip((unsigned char *)icmphdrout,
-                                    ICMP_SIZE);
+  icmphdrout->checksum= checksum_ip((unsigned char *)icmphdrout, ICMP_SIZE);
 #else
   icmphdrout->icmp_code= 0;
   icmphdrout->icmp_type= ICMP_ECHO;
   icmphdrout->icmp_id= getpid();
   icmphdrout->icmp_seq= 0;
-  icmphdrout->icmp_cksum= checksum_ip((unsigned char *)icmphdrout,
-                                    ICMP_SIZE);
+  icmphdrout->icmp_cksum= checksum_ip((unsigned char *)icmphdrout, ICMP_SIZE);
 #endif
   sout.sin_family= AF_INET;
   sout.sin_port= 0;
   memcpy(&sout.sin_addr, hp->h_addr, hp->h_length);
 
+  /* Get time of connection attempt beginning */
+  gettimeofday(&t1, NULL);
+
   do {
     n= sendto(s, (char *)icmphdrout, ICMP_SIZE, 0,
 	      (struct sockaddr *)&sout, sizeof(struct sockaddr));
   } while(n == -1 && errno == EINTR);
   
-  tv.tv_sec= timeout;
-  tv.tv_usec= 0;
-
   do {
 
     FD_ZERO(&rset);
@@ -693,7 +670,7 @@ int icmp_echo(const char *hostname, int timeout) {
     size= sizeof(struct sockaddr_in);
 
     do {
-      n= recvfrom(s, buf, STRLEN, 0, (struct sockaddr *)&sin, &size);
+      n= recvfrom(s, buf, STRLEN, 0, (struct sockaddr *)&sin, (socklen_t *)&size);
     } while(n == -1 && errno == EINTR);
     
     if(n < 0)
@@ -713,7 +690,12 @@ int icmp_echo(const char *hostname, int timeout) {
         (icmphdrin->icmp_seq == icmphdrout->icmp_seq) ) {
 #endif
 
-      rv= TRUE;
+      /* Get time of connection attempt finish */
+      gettimeofday(&t2, NULL);
+
+      /* Get the response time */
+      response= (double)(t2.tv_sec  - t1.tv_sec) +
+                (double)(t2.tv_usec - t1.tv_usec)/1000000;
       break;
 
     }
@@ -724,7 +706,7 @@ int icmp_echo(const char *hostname, int timeout) {
   FREE(icmphdrout);
   close_socket(s);
 
-  return rv;
+  return response;
  
 }
 
@@ -782,12 +764,13 @@ static int do_connect(int s, const struct sockaddr *addr,
  * Compute Internet Checksum for "count" bytes beginning at location "addr".
  * Based on RFC1071.
  */
-static unsigned short checksum_ip(unsigned char *addr, int count) {
+static unsigned short checksum_ip(unsigned char *_addr, int count) {
 
   register long sum= 0;
+  unsigned short *addr= (unsigned short *)_addr;
 
   while(count > 1) {
-    sum += *((unsigned short *)addr)++;
+    sum += *addr++;
     count -= 2;
   }
 
