@@ -32,7 +32,7 @@
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: net.h,v 1.22 2004/02/06 20:02:20 martinp Exp $
+ *  @version \$Id: net.h,v 1.24 2004/07/31 21:36:06 martinp Exp $
  *
  *  @file
  */
@@ -45,18 +45,6 @@
 #define NET_TIMEOUT 5
 
 
-/**
- * Test INET Socket connection to host on port with the specified
- * socket protocol type. The protocol should be either SOCK_STREAM or
- * SOCK_DGRAM.
- * @param hostname The host to connect to
- * @param port The portnumber to connect to
- * @param protocol The socket protocol to use
- * @return TRUE if the connect succeeded otherwise FALSE.
- */
-int check_connect(const char *hostname, int port, int protocol);
-
-
 /**
  * Check if the hostname resolves
  * @param hostname The host to check
@@ -204,8 +192,8 @@ int sock_read(int socket, void *buffer, int size, int timeout);
  * Create a ICMP socket against hostname, send echo and wait for response.
  * @param hostname The host to open a socket at
  * @param timeout If response will not come within timeout seconds abort
- * @return TRUE on succes, FALSE on error
+ * @return response time on succes, -1 on error
  */
-int icmp_echo(const char *hostname, int timeout);
+double icmp_echo(const char *hostname, int timeout);
 
 #endif
