@@ -20,15 +20,13 @@
 #ifndef MONIT_PROCESS_SYSDEP_H
 #define MONIT_PROCESS_SYSDEP_H
 
-int    get_process_info(ProcInfo_T);
-int    get_process_info_sysdep(ProcInfo_T);
+int init_process_info_sysdep(void);
+int init_proc_info_sysdep(void);
 
-int    init_process_info_sysdep(void);
-int    init_proc_info_sysdep(void);
-
-int    read_proc_file(char *, int, char *, int);
-int    getdatafromproc(int, ProcessTree_T *);
-int    getloadavg_sysdep (double *, int);
+int read_proc_file(char *, int, char *, int);
+int getloadavg_sysdep (double *, int);
+int used_system_memory_sysdep(SystemInfo_T *);
+int used_system_cpu_sysdep(SystemInfo_T *);
 
 double get_float_time(void);
 
@@ -39,4 +37,3 @@ int    connectchild(ProcessTree_T * , ProcessTree_T *);
 
 
 #endif
-
