@@ -70,6 +70,18 @@
 #include <procfs.h>
 #endif
 
+#ifdef HAVE_CF_H
+#include <cf.h>
+#endif
+
+#ifdef HAVE_SYS_CFGODM_H
+#include <sys/cfgodm.h>
+#endif
+
+#ifdef HAVE_SYS_CFGDB_H
+#include <sys/cfgdb.h>
+#endif
+
 
 #include "monitor.h"
 #include "process.h"
@@ -82,7 +94,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: sysdep_AIX.c,v 1.7 2004/02/29 22:24:45 martinp Exp $
+ *  @version \$Id: sysdep_AIX.c,v 1.11 2004/09/09 20:17:42 martinp Exp $
  *
  *  @file
  * most of the code is based on monitor-2.1.5 available on various sources
@@ -100,7 +112,17 @@ int nlistdone = 0;
 
 int init_process_info_sysdep(void) {
 
-  mem_kbyte_max = 0;
+  struct CuAt*      odmobj;
+  int               results;
+  
+  if (odm_initialize() == 0) {
+    odmobj = (struct CuAt*)getattr ("sys0", "realmem", 0, &results);
+    systeminfo.mem_kbyte_max = strtoull(odmobj->value, 0, 10);
+    odm_terminate();
+  } else {
+    mem_kbyte_max = 0;
+  }
+
   num_cpus= sysconf( _SC_NPROCESSORS_ONLN);
   return (getuid()==0);
 
@@ -112,69 +134,6 @@ double timestruc_to_tseconds(struct timeval t) {
 
 }
 
-
-int get_process_info_sysdep(ProcInfo_T p) {
-    
-  int nproc;
-  int i,rv;
-  struct userinfo user;
-  
-  memset(&user,0,sizeof(struct userinfo));
-  nproc=getproc(procs,NPROCS,sizeof(struct procinfo));
-  for(i=0;i<nproc;i++) {
-    
-    if(p->pid==procs[i].pi_pid) {
-      
-      if(procs[i].pi_stat==SZOMB) {
-
-empty:
-
-	p->cputime_prev= p->cputime = 0;
-	p->mem_kbyte= 0;
-	p->mem_percent= 0.0;
-	p->status_flag |= PROCESS_ZOMBIE;
- 
-     } else {
-
-       rv=getuser(&(procs[i]),sizeof(struct procinfo),
-		  &user,sizeof(struct userinfo));
-      
-       if(rv== -1) {
-
-	 goto empty;
-
-       }
-
-       p->mem_percent = user.ui_prm*10;
-       p->mem_kbyte = (user.ui_drss+user.ui_trss)*4;
-       
-       p->cputime_prev= p->cputime;
-       p->cputime= ( timestruc_to_tseconds(user.ui_ru.ru_utime) +
-		     timestruc_to_tseconds(user.ui_ru.ru_stime));
-       
-       if( include_children ) {
-	 p->cputime+= ( 
-		       timestruc_to_tseconds(user.ui_cru.ru_utime)+
-		       timestruc_to_tseconds(user.ui_cru.ru_stime));
-       }
-       /* first run ? */
-       if ( p->time_prev == 0.0 ) {
-	
-	 p->cputime_prev= p->cputime;
-
-       }       
-     }    
-      
-      return TRUE; 
-
-    }
-  }
-
-  return FALSE;
-    
-}
-
-
 /***************************************************************************
  *                      DATA CAPTURE FUNCTIONS                             *
  ***************************************************************************/
@@ -216,41 +175,6 @@ int getkmemdata(void *buf, int bufsize, caddr_t address) {
  */
 int getloadavg_sysdep (double *loadv, int nelem) {
 
-  /*
-  static int initted=0;
-  int avenrun[3];
-  static int no_avenrun_here = 0;
-  static struct nlist kernelnames[] = {
-    {"avenrun", 0, 0, 0, 0, 0},
-    {NULL, 0, 0, 0, 0, 0}
-  };
-  
-  if (no_avenrun_here)
-    return FALSE;
-  
-  if (!initted) {
-
-    initted = 1;
-    if (knlist(kernelnames, 1, sizeof(struct nlist)) == -1){
-
-      no_avenrun_here = 1;
-      return -1;
-
-    }
-
-  }
-
-  getkmemdata(&avenrun, sizeof(avenrun), (caddr_t) kernelnames->n_value);
-  if (nelem > 0)
-    loadv[0] = avenrun[0] / 65536.0;
-  if (nelem > 1)
-    loadv[1] = avenrun[1] / 65536.0;
-  if (nelem > 2)
-    loadv[2] = avenrun[2] / 65536.0;
-  
-  return TRUE;
-  */
-
   return getloadavg(loadv, nelem);
 
 }
@@ -289,23 +213,30 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
 
     if(procs[i].pi_stat==SZOMB) {
 
+      pt[i].cputime = 0;
       pt[i].mem_kbyte= 0;
-
-    } else {
+      pt[i].status_flag |= PROCESS_ZOMBIE;
+ 
+     } else {
 
       rv=getuser(&(procs[i]),sizeof(struct procinfo),
-		 &user,sizeof(struct userinfo));
-
+                 &user,sizeof(struct userinfo));
+      
       if(rv== -1) {
-
-	pt[i].mem_kbyte= 0;
-
+        
+        pt[i].cputime = 0;
+        pt[i].mem_kbyte= 0;
+        
       } else {
 
         pt[i].mem_kbyte = (user.ui_drss+user.ui_trss)*4;
-
+        pt[i].cputime= ( timestruc_to_tseconds(user.ui_ru.ru_utime) +
+                         timestruc_to_tseconds(user.ui_ru.ru_stime));
+        
       }
+      
     }    
+    
   }
 
   * reference = pt;
@@ -313,3 +244,25 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   return treesize;
 
 }
+
+
+/**
+ * This routine returns kbyte of real memory in use.
+ * @return: TRUE if successful, FALSE if failed (or not available)
+ */
+int used_system_memory_sysdep(SystemInfo_T *si) {
+
+  return FALSE;
+  
+}
+
+
+/**
+ * This routine returns system/user CPU time in use.
+ * @return: TRUE if successful, FALSE if failed (or not available)
+ */
+int used_system_cpu_sysdep(SystemInfo_T *si) {
+
+  return FALSE;
+  
+}
