@@ -30,7 +30,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: sysdep_UNKNOWN.c,v 1.4 2004/02/29 22:24:45 martinp Exp $
+ *  @version \$Id: sysdep_UNKNOWN.c,v 1.7 2004/09/09 20:17:42 martinp Exp $
  *
  *  @file
  */
@@ -38,19 +38,13 @@
 
 int init_process_info_sysdep(void) {
 
-  mem_kbyte_max = 0;
+  systeminfo.mem_kbyte_max = 0;
 
   return FALSE;
 
 }
 
 
-int get_process_info_sysdep(ProcInfo_T p) {
-    
-  return FALSE;
-    
-}
-
 /**
  * THIS IS JUST A DUMMY!!!
  *
@@ -88,3 +82,26 @@ int getloadavg_sysdep (double *loadv, int nelem) {
   return -1;
 
 }
+
+
+/**
+ * This routine returns kbyte of real memory in use.
+ * @return: TRUE if successful, FALSE if failed (or not available)
+ */
+int used_system_memory_sysdep(SystemInfo_T *si) {
+
+  return FALSE;
+
+}
+
+
+/**
+ * This routine returns system/user CPU time in use.
+ * @return: TRUE if successful, FALSE if failed (or not available)
+ */
+int used_system_cpu_sysdep(SystemInfo *si) {
+
+  return FALSE;
+  
+}
+
