@@ -63,7 +63,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Martin Pala, <martinp@tildeslash.com>
  *
- *  @version \$Id: dwp.c,v 1.10 2004/02/18 22:31:42 chopp Exp $
+ *  @version \$Id: dwp.c,v 1.11 2004/06/10 09:57:00 chopp Exp $
  *
  *  @file
  */
@@ -84,7 +84,7 @@ int check_dwp(Socket_T s) {
     return FALSE;
   }
   
-  if(socket_readln(s, buf, sizeof(buf)) <= 0) {
+  if(! socket_readln(s, buf, sizeof(buf))) {
     log("DWP: error receiving data -- %s\n", STRERROR);
     return FALSE;
   }
