@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -37,30 +37,29 @@
  * 
  *  @author Igor Homyakov, <homyakov@altlinux.ru>
  *
- *  @version \$Id: ssh.c,v 1.6 2003/10/25 19:17:14 hauk Exp $
+ *  @version \$Id: ssh.c,v 1.8 2004/01/29 17:52:12 martinp Exp $
  *
  *  @file
  */
 int check_ssh(Socket_T s) {
 
+  int n;
   char  buf[STRLEN];
 
   ASSERT(s);
     
-  if(socket_read(s, buf, sizeof(buf)) <= 0) {
+  if((n= socket_read(s, buf, sizeof(buf))) <= 0) {
     log("SSH: error receiving identification string -- %s\n", STRERROR);
     return FALSE;
   }
-
-  chomp(buf);
   
-  if(strncmp(buf, "SSH-", 4) != 0) {
+  if(! starts_with(buf, "SSH-")) {
     log("SSH: protocol error %s\n", buf);
     return FALSE;
   }
 
   /* send identification string back to server */
-  if(socket_write(s, buf, strlen(buf)) <= 0) {
+  if(socket_write(s, buf, n) <= 0) {
     log("SSH: error sending identification string -- %s\n", STRERROR);
     return FALSE;
   }
