@@ -66,7 +66,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: sendmail.c,v 1.29 2004/02/18 22:31:42 chopp Exp $
+ *  @version \$Id: sendmail.c,v 1.30 2004/07/15 12:20:12 chopp Exp $
  *
  *  @file
  */
@@ -79,6 +79,7 @@ typedef struct {
   Socket_T socket;
   sigjmp_buf error;
   const char *server;
+  int port;
   char localhost[STRLEN];
 } SendMail_T;
 
@@ -219,17 +220,18 @@ static void open_server(SendMail_T *S) {
   MailServer_T mta= Run.mailservers;
 
   S->server= mta?mta->host:LOCALHOST;
+  S->port=   mta?mta->port:SMTP_PORT;
 
   do {
-    S->socket= socket_create_t(S->server, SMTP_PORT, SOCKET_TCP, FALSE,
+    S->socket= socket_create_t(S->server, S->port, SOCKET_TCP, FALSE,
 			       Run.mailserver_timeout);
     if(S->socket)
 	break;
-    log("Cannot open a connection to the mailserver '%s' -- %s\n",
-	S->server, STRERROR);
+    log("Cannot open a connection to the mailserver '%s:%i' -- %s\n",
+	S->server, S->port, STRERROR);
     if(mta && (mta= mta->next)) {
       S->server= mta->host;
-      log("Trying the next mail server '%s'\n", S->server);
+      log("Trying the next mail server '%s:%i'\n", S->server, S->port);
       continue;
     } else {
       log("No mail servers are available, aborting alert\n");
