@@ -50,7 +50,7 @@
  * Implementation of the socket interface.
  * 
  * @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- * @version \$Id: socket.c,v 1.34 2004/02/18 14:05:54 hauk Exp $
+ * @version \$Id: socket.c,v 1.35 2004/05/15 20:03:53 martinp Exp $
  * @file
  */
 
@@ -432,7 +432,7 @@ int socket_write(Socket_T S, void *b, int size) {
     return -1;
   }
   
-  return  (int) p - (int) b;
+  return  (int)(p - b);
 
 }
 
@@ -473,7 +473,7 @@ int socket_read(Socket_T S, void *b, int size) {
     return -1;
   }
   
-  return (int) p - (int) b;
+  return (int)(p - b);
  
 }
  
