@@ -77,7 +77,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Peter Holdaway <pholdaway@technocom-wireless.com>
  *
- *  @version \$Id: spawn.c,v 1.40 2004/03/10 23:04:42 hauk Exp $
+ *  @version \$Id: spawn.c,v 1.42 2004/08/09 19:57:51 martinp Exp $
  *  @file
  */
 
@@ -271,14 +271,13 @@ static void set_monit_environment(Service_T s,
     snprintf(buf, STRLEN, "MONIT_PROCESS_PID=%d", is_process_running(s));
     push_monit_environment(buf, e);
 
-    snprintf(buf, STRLEN, "MONIT_PROCESS_MEMORY=%ld", s->procinfo->mem_kbyte);
+    snprintf(buf, STRLEN, "MONIT_PROCESS_MEMORY=%ld", s->inf->mem_kbyte);
     push_monit_environment(buf, e);
 
-    snprintf(buf, STRLEN, "MONIT_PROCESS_CHILDREN=%d", s->procinfo->children);
+    snprintf(buf, STRLEN, "MONIT_PROCESS_CHILDREN=%d", s->inf->children);
     push_monit_environment(buf, e);
 
-    snprintf(buf, STRLEN, "MONIT_PROCESS_CPU_PERCENT=%d", 
-	     s->procinfo->cpu_percent);
+    snprintf(buf, STRLEN, "MONIT_PROCESS_CPU_PERCENT=%d", s->inf->cpu_percent);
     push_monit_environment(buf, e);
   }
 
