@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -150,7 +150,7 @@ static void handle_ssl_error(char *);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: ssl.c,v 1.34 2003/10/25 19:17:11 hauk Exp $
+ *  @version \$Id: ssl.c,v 1.36 2004/02/18 22:31:42 chopp Exp $
  *
  *  @file
  */
@@ -295,7 +295,9 @@ int update_ssl_cert_data(ssl_connection *ssl) {
 
   } 
 
+  /* RATS: ignore */ /* buffer is dynamically allocated */
   ssl->cert_issuer=X509_NAME_oneline (X509_get_issuer_name (ssl->cert),0,0);
+  /* RATS: ignore */ /* buffer is dynamically allocated */
   ssl->cert_subject=X509_NAME_oneline (X509_get_subject_name (ssl->cert),0,0);
   X509_digest(ssl->cert,EVP_md5(), md5, &ssl->cert_md5_len);
   ssl->cert_md5= (unsigned char *) xstrdup((char *) md5);
@@ -1648,7 +1650,9 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx) {
   char subject[STRLEN];
   X509_OBJECT found_cert;
 
-  X509_NAME_oneline(X509_get_subject_name(ctx->current_cert), subject, STRLEN);
+  /* RATS: ignore */ /* buffer size is limited by STRLEN */
+  X509_NAME_oneline(X509_get_subject_name(ctx->current_cert), subject,
+                    STRLEN-1);
 
   if(!preverify_ok) {
 
@@ -1689,7 +1693,9 @@ static int verify_callback_noclientcert(int preverify_ok,
 
   char subject[STRLEN];
 
-  X509_NAME_oneline(X509_get_subject_name(ctx->current_cert), subject, STRLEN);
+  /* RATS: ignore */ /* buffer size is limited by STRLEN-1 */
+  X509_NAME_oneline(X509_get_subject_name(ctx->current_cert), subject,
+                    STRLEN-1);
 
   if(!preverify_ok) {
 
