@@ -63,7 +63,7 @@
  *  file, if not set, the default is ~/.monit.state.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @version \$Id: state.c,v 1.18 2004/02/14 22:26:43 hauk Exp $
+ *  @version \$Id: state.c,v 1.22 2004/08/04 22:02:09 martinp Exp $
  *  @file
  */
 
@@ -76,14 +76,12 @@
  * when handling the state.
  */
 typedef struct mystate {
-  char name[STRLEN];   
-  int  mode;       
-  int  nstart;     
-  int  ncycle;     
-  int  do_monitor;
-  int  has_permission_error;
-  int  has_uid_error;
-  int  has_gid_error;
+  char               name[STRLEN];
+  int                mode;
+  int                nstart;
+  int                ncycle;
+  int                monitor;
+  unsigned long long error;
 } State_T;
 
 
@@ -268,10 +266,8 @@ static void clone_state(Service_T service, State_T *state) {
   state->mode= service->mode;
   state->nstart= service->nstart;
   state->ncycle= service->ncycle;
-  state->do_monitor= service->do_monitor;
-  state->has_permission_error= service->perm?service->perm->has_error:FALSE;
-  state->has_uid_error= service->uid?service->uid->has_error:FALSE;
-  state->has_gid_error= service->gid?service->gid->has_error:FALSE;
+  state->monitor= service->monitor;
+  state->error= service->error;
 
 }
 
@@ -281,12 +277,7 @@ static void update_service_state(Service_T service, State_T *state) {
   service->mode= state->mode;
   service->nstart= state->nstart;
   service->ncycle= state->ncycle;
-  service->do_monitor= state->do_monitor;
-  if(service->perm)
-    service->perm->has_error= state->has_permission_error;
-  if(service->uid)
-    service->uid->has_error= state->has_uid_error;
-  if(service->gid)
-    service->gid->has_error= state->has_gid_error;
+  service->monitor= state->monitor;
+  service->error= state->error;
   
 }
