@@ -98,7 +98,8 @@
 /* Private prototypes */
 static char x2c(char *hex);
 static char *is_str_defined(char *);
-static int is_unsafe(unsigned char *c);
+static int is_unsafe(unsigned char *);
+static void printevents(unsigned int);
 
 
 /**
@@ -109,7 +110,7 @@ static int is_unsafe(unsigned char *c);
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.c,v 1.115 2004/02/29 22:24:44 martinp Exp $
+ *  @version \$Id: util.c,v 1.130 2004/08/18 07:03:43 martinp Exp $
  *  @file
  */
 
@@ -566,37 +567,7 @@ void printrunlist() {
     for(list= Run.maillist; list; list= list->next) {
       printf(" %-18s = %s\n", "Alert mail to", is_str_defined(list->to));
       printf(" %-18s = ", "Alert on");
-      if(list->events == (~((unsigned int)0))) {
-        printf("All events");
-      } else {
-        if(IS_EVENT_SET(list->events, EVENT_START))
-	  printf("Start ");
-        if(IS_EVENT_SET(list->events, EVENT_STOP))
-	  printf("Stop ");
-        if(IS_EVENT_SET(list->events, EVENT_RESTART))
-	  printf("Restart ");
-        if(IS_EVENT_SET(list->events, EVENT_CHECKSUM))
-	  printf("Checksum ");
-        if(IS_EVENT_SET(list->events, EVENT_RESOURCE))
-	  printf("Resource ");
-        if(IS_EVENT_SET(list->events, EVENT_TIMEOUT))
-	  printf("Timeout ");
-        if(IS_EVENT_SET(list->events, EVENT_TIMESTAMP))
-	  printf("Timestamp ");
-        if(IS_EVENT_SET(list->events, EVENT_SIZE))
-	  printf("Size ");
-        if(IS_EVENT_SET(list->events, EVENT_CONNECTION))
-	  printf("Connection ");
-        if(IS_EVENT_SET(list->events, EVENT_PERMISSION))
-	  printf("Permission ");
-        if(IS_EVENT_SET(list->events, EVENT_UID))
-	  printf("Uid ");
-        if(IS_EVENT_SET(list->events, EVENT_GID))
-	  printf("Gid ");
-        if(IS_EVENT_SET(list->events, EVENT_UNMONITOR))
-	  printf("Unmonitor ");
-      }
-      printf("\n");
+      printevents(list->events);
     }
   }
 
@@ -619,14 +590,16 @@ void printservice(Service_T s) {
   Timestamp_T t;
   Size_T sl;
   Dependant_T d;
+  char string[STRLEN];
 
   ASSERT(s);
  
-  printf("%-21s = %s\n", servicenames[s->type], s->name);
+  snprintf(string, STRLEN, "%s Name", servicetypes[s->type]);
+  printf("%-21s = %s\n", string, s->name);
   printf(" %-20s = %s\n", "Group", is_str_defined(s->group));
   if(s->type == TYPE_PROCESS)
     printf(" %-20s = %s\n", "Pid file", s->path);
-  else if(s->type != TYPE_REMOTE)
+  else if(s->type != TYPE_HOST)
     printf(" %-20s = %s\n", "Path", s->path);
   printf(" %-20s = %s\n", "Monitoring mode", modenames[s->mode]);
   if(s->start) {
@@ -650,65 +623,87 @@ void printservice(Service_T s) {
     if(d->dependant != NULL)
       printf(" %-20s = %s\n", "Depends on Service", d->dependant);
 
-  if(s->checksum) {
-    switch (s->checksum->type) {
-    case HASH_MD5:
-        printf(" %-20s = if failed MD5(%s) then %s\n",
-               "Checksum", s->checksum->hash,
-               actionnames[s->checksum->action]);
-        break;
-    case HASH_SHA1:
-        printf(" %-20s = if failed SHA1(%s) then %s\n",
-               "Checksum", s->checksum->hash,
-               actionnames[s->checksum->action]);
-        break;
-    default:
-        printf(" %-20s = if failed UNKNOWN(%s) then %s\n",
-               "Checksum", s->checksum->hash,
-               actionnames[s->checksum->action]);
-        break;
-    }
+  if(s->checksum && s->checksum->action) {
+    Checksum_T cs= s->checksum;
+    EventAction_T a= cs->action;
+
+    if(cs->test_changes)
+      printf(" %-20s = if changed %s then %s\n",
+        "Checksum", checksumnames[cs->type], actionnames[a->failed->id]);
+    else
+      printf(" %-20s = if failed %s(%s) then %s else if recovered %s\n",
+        "Checksum", cs->hash, checksumnames[cs->type],
+        actionnames[a->failed->id], actionnames[a->passed->id]);
   }
   
-  if(s->perm)
-    printf(" %-20s = if failed %o then %s\n",
-           "Permission", s->perm->perm, actionnames[s->perm->action]);
+  if(s->perm && s->perm->action) {
+    EventAction_T a= s->perm->action;
 
-  if(s->uid)
-    printf(" %-20s = if failed %d then %s\n",
-           "UID", (int)s->uid->uid, actionnames[s->uid->action]);
+    printf(" %-20s = if failed %o then %s else if recovered then %s\n",
+      "Permission", s->perm->perm, actionnames[a->failed->id],
+      actionnames[a->passed->id]);
+  }
 
-  if(s->gid)
-    printf(" %-20s = if failed %d then %s\n",
-           "GID", (int)s->gid->gid, actionnames[s->gid->action]);
+  if(s->uid && s->uid->action) {
+    EventAction_T a= s->uid->action;
+
+    printf(" %-20s = if failed %d then %s else if recovered then %s\n",
+      "UID", (int)s->uid->uid, actionnames[a->failed->id],
+      actionnames[a->passed->id]);
+  }
+
+  if(s->gid && s->gid->action) {
+    EventAction_T a= s->gid->action;
+
+    printf(" %-20s = if failed %d then %s else if recovered then %s\n",
+      "GID", (int)s->gid->gid, actionnames[a->failed->id],
+      actionnames[a->passed->id]);
+  }
+
+  if(s->icmplist)
+    for(i= s->icmplist; i; i= i->next) {
+      EventAction_T a= i->action;
+
+      printf(" %-20s = if failed %s with timeout %d seconds then %s "
+        "else if recovered then %s\n",
+        "ICMP", icmpnames[i->type], i->timeout, actionnames[a->failed->id],
+        actionnames[a->passed->id]);
+    }
 
   if(s->portlist) {
     
     for(n= s->portlist; n; n= n->next) {
+      EventAction_T a= n->action;
       
       if(n->family == AF_INET) {
 	
 	if(n->SSL.use_ssl) {
 	  
-	  printf(" %-20s = %s:%d%s [protocol %s via SSL] with timeout %d seconds\n",
-	        "Host:Port", n->hostname, n->port, n->request?n->request:"",
-		 n->protocol->name, n->timeout);
+	  printf(" %-20s = if failed %s:%d%s [protocol %s via SSL] with "
+                "timeout %d seconds then %s else if recovered then %s\n",
+	        "Port", n->hostname, n->port, n->request?n->request:"",
+		 n->protocol->name, n->timeout, actionnames[a->failed->id],
+                 actionnames[a->passed->id]);
 	  
 	  if(n->SSL.certmd5 != NULL)
 	      printf(" %-20s = %s\n", "Server cert md5 sum", n->SSL.certmd5);
 	  
         } else {
 
-	  printf(" %-20s = %s:%d%s [protocol %s] with timeout %d seconds\n",
-	         "Host:Port", n->hostname, n->port, n->request?n->request:"",
-		 n->protocol->name, n->timeout);
+	  printf(" %-20s = if failed %s:%d%s [protocol %s] with timeout %d "
+                 "seconds then %s else if recovered then %s\n",
+	         "Port", n->hostname, n->port, n->request?n->request:"",
+		 n->protocol->name, n->timeout, actionnames[a->failed->id],
+                 actionnames[a->passed->id]);
 	  
 	}
 	
       } else if(n->family == AF_UNIX) {
 	
-        printf(" %-20s = %s [protocol %s] with timeout %d seconds\n",
-	       "Unix Socket", n->pathname, n->protocol->name, n->timeout);
+        printf(" %-20s = if failed %s [protocol %s] with timeout %d seconds "
+               "then %s else if recovered then %s\n",
+	       "Unix Socket", n->pathname, n->protocol->name, n->timeout,
+               actionnames[a->failed->id], actionnames[a->passed->id]);
 	
       }
       
@@ -716,117 +711,140 @@ void printservice(Service_T s) {
     
   }
   
-  if(s->icmplist)
-    for(i= s->icmplist; i; i= i->next)
-      printf(" %-20s = if failed %s with timeout %d seconds then %s\n",
-             "ICMP", icmpnames[i->type], i->timeout, actionnames[i->action]);
-
   for(t= s->timestamplist; t; t= t->next) {
-    
-    if(t->test_changes) {
+    EventAction_T a= t->action;
+
+    if(t->test_changes)
       printf(" %-20s = if changed then %s\n",
-	     "Timestamp",
-	     actionnames[t->action]);
-    } else {
-      printf(" %-20s = if %s %d second(s) then %s\n",
-	     "Timestamp",
-	     operatornames[t->operator],
-	     t->time,
-	     actionnames[t->action]);
-    }
+        "Timestamp", actionnames[a->failed->id]);
+    else
+      printf(" %-20s = if %s %d second(s) then %s else if recovered then %s\n",
+        "Timestamp", operatornames[t->operator], t->time,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
     
   }
 
   for(sl= s->sizelist; sl; sl= sl->next) {
+    EventAction_T a= sl->action;
     
-    if(sl->test_changes) {
+    if(sl->test_changes)
       printf(" %-20s = if changed then %s\n",
-	     "Size",
-	     actionnames[sl->action]);
-    } else {
-      printf(" %-20s = if %s %lu byte(s) then %s\n",
-	     "Size",
-	     operatornames[sl->operator],
-	     sl->size,
-	     actionnames[sl->action]);
-    }
+        "Size", actionnames[a->failed->id]);
+    else
+      printf(" %-20s = if %s %lu byte(s) then %s else if recovered then %s\n",
+        "Size", operatornames[sl->operator], sl->size,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
     
   }
 
   for(dl= s->devicelist; dl; dl= dl->next) {
+    EventAction_T a= dl->action;
     
     if(dl->resource == RESOURCE_ID_INODE) {
-
-      printf(" %-20s = if %s %ld %s then %s\n",
-           "Inodes usage limit",
-           operatornames[dl->operator],
-           (dl->limit_absolute > -1)?dl->limit_absolute:dl->limit_percent,
-           (dl->limit_absolute > -1)?"":"%",
-           actionnames[dl->action]);
-
+	    
+      printf(" %-20s = if %s %ld %s then %s else if recovered then %s\n",
+        "Inodes usage limit",
+        operatornames[dl->operator],
+        (dl->limit_absolute > -1)?dl->limit_absolute:dl->limit_percent,
+        (dl->limit_absolute > -1)?"":"%",
+        actionnames[a->failed->id], actionnames[a->passed->id]);
+      
     } else if(dl->resource == RESOURCE_ID_SPACE) {
-
-      printf(" %-20s = if %s %ld %s then %s\n",
-           "Space usage limit",
-           operatornames[dl->operator],
-           (dl->limit_absolute > -1)?dl->limit_absolute:dl->limit_percent,
-           (dl->limit_absolute > -1)?"blocks":"%",
-           actionnames[dl->action]);
+	    
+      printf(" %-20s = if %s %ld %s then %s else if recovered then %s\n",
+        "Space usage limit",
+        operatornames[dl->operator],
+        (dl->limit_absolute > -1)?dl->limit_absolute:dl->limit_percent,
+        (dl->limit_absolute > -1)?"blocks":"%",
+        actionnames[a->failed->id], actionnames[a->passed->id]);
 
     }
     
   }
 
   for(q= s->resourcelist; q; q= q->next) {
+    EventAction_T a= q->action;
 
     switch(q->resource_id) {
 
     case RESOURCE_ID_CPU_PERCENT: 
 
-      printf(" %-20s = if %s %.1f%% for %d cycle(s) then %s\n", 
-	     "CPU usage limit", 
-	     operatornames[q->operator], 
-	     q->limit/10.0, q->max_cycle, actionnames[q->action]);
+      printf(" %-20s = if %s %.1f%% for %d cycle(s) then %s "
+        "else if recovered then %s\n", 
+        "CPU usage limit", 
+        operatornames[q->operator], q->limit/10.0, q->max_cycle,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_MEM_PERCENT: 
 
-      printf(" %-20s = if %s %.1f%% for %d cycle(s) then %s\n", 
-	     "Memory usage limit", 
-	     operatornames[q->operator], q->limit/10.0, q->max_cycle, 
-	     actionnames[q->action]);
+      printf(" %-20s = if %s %.1f%% for %d cycle(s) then %s "
+        "else if recovered then %s\n", 
+        "Memory usage limit", 
+        operatornames[q->operator], q->limit/10.0, q->max_cycle,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_MEM_KBYTE: 
 
-      printf(" %-20s = if %s %ldkB for %d cycle(s) then %s\n", 
-	     "Memory amount limit", 
-	     operatornames[q->operator], q->limit, q->max_cycle, 
-	     actionnames[q->action]);
+      printf(" %-20s = if %s %ldkB for %d cycle(s) then %s "
+        "else if recovered then %s\n", 
+        "Memory amount limit", 
+        operatornames[q->operator], q->limit, q->max_cycle,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_LOAD1: 
 
-      printf(" %-20s = if %s %.1f for %d cycle(s) then %s\n", 
-	     "Load avg. (1min)", 
-	     operatornames[q->operator], q->limit/10.0, q->max_cycle, 
-	     actionnames[q->action]);
+      printf(" %-20s = if %s %.1f for %d cycle(s) then %s "
+        "else if recovered then %s\n", 
+        "Load avg. (1min)", 
+        operatornames[q->operator], q->limit/10.0, q->max_cycle, 
+        actionnames[a->failed->id], actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_LOAD5: 
 
-      printf(" %-20s = if %s %.1f for %d cycle(s) then %s\n", 
-	     "Load avg. (5min)", 
-	     operatornames[q->operator], q->limit/10.0, q->max_cycle, 
-	     actionnames[q->action]);
+      printf(" %-20s = if %s %.1f for %d cycle(s) then %s "
+        "else if recovered then %s\n", 
+        "Load avg. (5min)", 
+        operatornames[q->operator], q->limit/10.0, q->max_cycle, 
+        actionnames[a->failed->id], actionnames[a->passed->id]);
       break;
 
     case RESOURCE_ID_LOAD15: 
 
-      printf(" %-20s = if %s %.1f for %d cycle(s) then %s\n", 
-	     "Load avg. (15min)", 
-	     operatornames[q->operator], q->limit/10.0, q->max_cycle, 
-	     actionnames[q->action]);
+      printf(" %-20s = if %s %.1f for %d cycle(s) then %s "
+        "else if recovered then %s\n", 
+        "Load avg. (15min)", 
+        operatornames[q->operator], q->limit/10.0, q->max_cycle,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
+      break;
+
+    case RESOURCE_ID_CHILDREN:
+
+      printf(" %-20s = If %s %ld for %d cycle(s) then %s "
+        "else if recovered then %s",
+        "Children", operatornames[q->operator], q->limit, q->max_cycle,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
+      break;
+
+    case RESOURCE_ID_TOTAL_MEM_KBYTE:
+
+      printf(" %-20s = If %s %ld for %d cycle(s) then %s "
+        "else if recovered then %s",
+        "Memory amount limit (incl. children)",
+        operatornames[q->operator], q->limit, q->max_cycle,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
+      break;
+
+    case RESOURCE_ID_TOTAL_MEM_PERCENT:
+
+      printf(" %-20s = If %s %.1f%% for %d cycle(s) then %s "
+        "else if recovered then %s",
+        "Memory usage limit (incl. children)",
+        operatornames[q->operator], q->limit/10.0, q->max_cycle,
+        actionnames[a->failed->id], actionnames[a->passed->id]);
       break;
 
     }    
@@ -835,51 +853,18 @@ void printservice(Service_T s) {
   if(s->def_every)
     printf(" %-20s = Check service every %d cycles\n", "Every", s->every);
   
-
-  if(s->def_timeout)
-    printf(" %-20s = Do timeout if %d restart within %d cycles\n",
-	   "Timeout", s->to_start, s->to_cycle);
+  if(s->def_timeout && s->action_TIMEOUT) {
+    EventAction_T a= s->action_TIMEOUT;
+    printf(" %-20s = If %d restart within %d cycles then %s "
+      "else if recovered then %s\n",
+      "Timeout", s->to_start, s->to_cycle, actionnames[a->failed->id],
+      actionnames[a->passed->id]);
+  }
 
   for(r= s->maillist; r; r= r->next) {
-    
     printf(" %-20s = %s\n", "Alert mail to", is_str_defined(r->to));
     printf(" %-20s = ", "Alert on");
-
-    if(r->events == (~((unsigned int)0))) {
-      
-      printf("All events");
-
-    } else {
-      if(IS_EVENT_SET(r->events, EVENT_START))
-	  printf("Start ");
-      if(IS_EVENT_SET(r->events, EVENT_STOP))
-	  printf("Stop ");
-      if(IS_EVENT_SET(r->events, EVENT_RESTART))
-	  printf("Restart ");
-      if(IS_EVENT_SET(r->events, EVENT_CHECKSUM))
-	  printf("Checksum ");
-      if(IS_EVENT_SET(r->events, EVENT_RESOURCE))
-	  printf("Resource ");
-      if(IS_EVENT_SET(r->events, EVENT_TIMEOUT))
-	  printf("Timeout ");
-      if(IS_EVENT_SET(r->events, EVENT_TIMESTAMP))
-	  printf("Timestamp ");
-      if(IS_EVENT_SET(r->events, EVENT_SIZE))
-	  printf("Size ");
-      if(IS_EVENT_SET(r->events, EVENT_CONNECTION))
-	  printf("Connection ");
-      if(IS_EVENT_SET(r->events, EVENT_PERMISSION))
-	  printf("Permission ");
-      if(IS_EVENT_SET(r->events, EVENT_UID))
-	  printf("Uid ");
-      if(IS_EVENT_SET(r->events, EVENT_GID))
-	  printf("Gid ");
-      if(IS_EVENT_SET(r->events, EVENT_UNMONITOR))
-	  printf("Unmonitor ");
-    }
-    
-    printf("\n");
-    
+    printevents(r->events);
   }
 
   printf("\n");
@@ -908,6 +893,83 @@ void printservicelist() {
   
 }
 
+/**
+ * Print file hashes from stdin or file
+ */
+void printhash(char *filename) {
+  
+  unsigned char buf[STRLEN], buf2[STRLEN];
+  FILE * fhandle;
+  int fresult;
+  int i;
+
+  if (filename == NULL) {
+    
+    fhandle = stdin;
+    
+  } else {
+    
+    fhandle = fopen(filename, "r");
+
+    if ( fhandle == NULL ) {
+
+      goto fileerror;
+      
+    }
+  }
+  
+  fresult=sha_md5_stream(fhandle, buf, buf2);
+
+  if(fresult) {
+
+    goto fileerror;
+
+  }
+
+  if (filename==NULL) {
+    
+      printf("SHA1(stdin) = ");
+      
+  } else {
+    
+      printf("SHA1(%s) = ", filename);
+      
+      fclose(fhandle);
+  }
+      
+  for(i= 0; i < 20; ++i) {
+    
+      printf("%02x", buf[i]);
+
+  }
+  
+  if (filename==NULL) {
+    
+      printf("\nMD5(stdin)  = ");
+      
+  } else {
+    
+      printf("\nMD5(%s)  = ", filename);
+
+  }
+  
+  for(i= 0; i < 16; ++i) {
+    
+      printf("%02x", buf2[i]);
+      
+  }
+  
+  printf("\n");
+  
+  return;
+
+fileerror:
+  
+  printf("monit: %s: %s\n", filename, strerror(errno));
+  
+  exit(1);
+
+}
 
 /**
  * Open and read the pid from the given pidfile.
@@ -976,7 +1038,7 @@ int is_process_running(Service_T s) {
     if( (getpgid(pid) > -1) || (errno == EPERM) )
       return pid;
 
-  memset(s->procinfo, 0, sizeof *(s->procinfo));
+  reset_procinfo(s);
 
   return FALSE;
   
@@ -1009,12 +1071,12 @@ char *get_RFC822date(long *date) {
 
 
 /**
- * Compute an uptime string for a process based on the ctime
- * from the pidfile. The caller must free the returned string.
+ * Compute an uptime for a process based on the ctime
+ * from the pidfile.
  * @param pidfile A process pidfile
- * @return an uptime string
+ * @return an uptime
  */
-char *get_process_uptime(char *pidfile, char *sep) {
+time_t get_process_uptime(char *pidfile) {
 
   time_t ctime;
 
@@ -1025,11 +1087,11 @@ char *get_process_uptime(char *pidfile, char *sep) {
     time_t now= time(&now);
     time_t since= now-ctime;
 
-    return get_uptime(since, sep);
+    return since;
 
   }
 
-  return xstrdup("");
+  return (time_t)-1;
 
 }
 
@@ -1054,6 +1116,9 @@ char *get_uptime(time_t delta, char *sep) {
 
   *buf= 0;
 
+  if(delta < 0)
+    return(xstrdup(""));
+
   if((rest_d= delta/day)>0) {
     p+= snprintf(p, STRLEN-(p-buf), "%ldd%s", rest_d,sep);
     delta-= rest_d*day;
@@ -1142,51 +1207,6 @@ char *get_checksum(char *file, int hashtype) {
 }
 
 
-/**
- * @param file A file to open and compute a checksum for
- * @param sum A previous checksum computed for this file
- * @return TRUE if computation of hash on the given file, is equal
- * to the given sum, otherwise return FALSE. FALSE is also returned if
- * *hash*_stream fails.
- */
-int check_hash(char *file, char *sum, int hashtype) {
-
-  char *newSum;
-
-  ASSERT(file);
-  ASSERT(sum);
-
-  newSum= get_checksum(file, hashtype);
-
-
-  if(newSum) {
-    
-    int rv;
-    
-    switch(hashtype) {
-    case HASH_MD5:
-        rv= (!strncmp(sum, newSum, 32));
-        break;
-        
-    case HASH_SHA1:
-        rv= (!strncmp(sum, newSum, 40));
-        break;
-
-    default:
-        rv= FALSE;
-    }
-    
-    FREE(newSum);
-    
-    return (rv);
-    
-  }
-
-  return FALSE;
-  
-}
-
-
 /**
  * Escape an uri string converting unsafe characters to a hex (%xx)
  * representation.  The caller must free the returned string.
@@ -1312,11 +1332,26 @@ char *format(const char *s, va_list ap, long *len) {
   int size= STRLEN;
   char *buf= xmalloc(size);
 
+#ifdef HAVE_VA_COPY
+  va_list ap_copy;
+#endif
+
   ASSERT(s);
-    
+   
   while(TRUE) {
+
+#ifdef HAVE_VA_COPY
+    va_copy(ap_copy, ap);
       
+    n= vsnprintf(buf, size, s, ap_copy);
+
+    va_end(ap_copy);
+
+#else
+
     n= vsnprintf(buf, size, s, ap);
+
+#endif
     
     if(n > -1 && n < size)
         break;
@@ -1331,7 +1366,7 @@ char *format(const char *s, va_list ap, long *len) {
   }
 
   *len= n;
-  
+
   return buf;
 
 }
@@ -1466,6 +1501,91 @@ int compare_user_credentials(char *uname, char *outside) {
 }
 
 
+/* Compute SHA1 and MD5 message digests simultaneously for bytes read
+   from STREAM (suitable for stdin, which is not always rewindable).
+   The resulting message digest numbers will be written into the first bytes
+   of resblock buffers.  */
+int sha_md5_stream (FILE *stream, void *sha_resblock, void *md5_resblock)
+{
+/* Important: HASHBLOCKSIZE must be a multiple of 64.  */
+#define HASHBLOCKSIZE 4096
+  struct sha_ctx ctx_sha;
+  struct md5_ctx ctx_md5;
+  char buffer[HASHBLOCKSIZE + 72];
+  size_t sum;
+
+  /* Initialize the computation contexts.  */
+  sha_init_ctx (&ctx_sha);
+  md5_init_ctx (&ctx_md5);
+
+  /* Iterate over full file contents.  */
+  while (1)
+    {
+      /* We read the file in blocks of HASHBLOCKSIZE bytes.  One call of the
+	 computation function processes the whole buffer so that with the
+	 next round of the loop another block can be read.  */
+      size_t n;
+      sum = 0;
+
+      /* Read block.  Take care for partial reads.  */
+      while (1)
+	{
+	  n = fread (buffer + sum, 1, HASHBLOCKSIZE - sum, stream);
+
+	  sum += n;
+
+	  if (sum == HASHBLOCKSIZE)
+	    break;
+
+	  if (n == 0)
+	    {
+	      /* Check for the error flag IFF N == 0, so that we don't
+		 exit the loop after a partial read due to e.g., EAGAIN
+		 or EWOULDBLOCK.  */
+	      if (ferror (stream))
+		return 1;
+	      goto process_partial_block;
+	    }
+
+	  /* We've read at least one byte, so ignore errors.  But always
+	     check for EOF, since feof may be true even though N > 0.
+	     Otherwise, we could end up calling fread after EOF.  */
+	  if (feof (stream))
+	    goto process_partial_block;
+	}
+
+      /* Process buffer with HASHBLOCKSIZE bytes.  Note that
+			HASHBLOCKSIZE % 64 == 0
+       */
+      sha_process_block (buffer, HASHBLOCKSIZE, &ctx_sha);
+      md5_process_block (buffer, HASHBLOCKSIZE, &ctx_md5);
+    }
+
+process_partial_block:
+
+  /* Process any remaining bytes.  */
+  if (sum > 0) {
+    sha_process_bytes (buffer, sum, &ctx_sha);
+    md5_process_bytes (buffer, sum, &ctx_md5);
+  }
+
+  /* Construct result in desired memory.  */
+  sha_finish_ctx (&ctx_sha, sha_resblock);
+  md5_finish_ctx (&ctx_md5, md5_resblock);
+  return 0;
+}
+
+
+/**
+  * Reset process information structure
+  */
+void reset_procinfo(Service_T s) {
+
+  memset(s->inf, 0, sizeof *(s->inf));
+
+}
+
+
 /* ----------------------------------------------------------------- Private */
 
 
@@ -1523,3 +1643,46 @@ static char x2c(char *hex) {
   
 }
 
+
+/**
+ * Print registered events list
+ */
+static void printevents(unsigned int events) {
+
+  if(events == (~((unsigned int)0))) {
+    printf("All events");
+  } else {
+    if(IS_EVENT_SET(events, EVENT_CHANGED))
+      printf("Change ");
+    if(IS_EVENT_SET(events, EVENT_CHECKSUM))
+      printf("Checksum ");
+    if(IS_EVENT_SET(events, EVENT_CONNECTION))
+      printf("Connection ");
+    if(IS_EVENT_SET(events, EVENT_DATA))
+      printf("Data ");
+    if(IS_EVENT_SET(events, EVENT_EXEC))
+      printf("Exec ");
+    if(IS_EVENT_SET(events, EVENT_GID))
+      printf("Gid ");
+    if(IS_EVENT_SET(events, EVENT_ICMP))
+      printf("Icmp ");
+    if(IS_EVENT_SET(events, EVENT_INVALID))
+      printf("Invalid ");
+    if(IS_EVENT_SET(events, EVENT_NONEXIST))
+      printf("Nonexist ");
+    if(IS_EVENT_SET(events, EVENT_PERMISSION))
+      printf("Permission ");
+    if(IS_EVENT_SET(events, EVENT_RESOURCE))
+      printf("Resource ");
+    if(IS_EVENT_SET(events, EVENT_SIZE))
+      printf("Size ");
+    if(IS_EVENT_SET(events, EVENT_TIMEOUT))
+      printf("Timeout ");
+    if(IS_EVENT_SET(events, EVENT_TIMESTAMP))
+    printf("Timestamp ");
+    if(IS_EVENT_SET(events, EVENT_UID))
+      printf("Uid ");
+  }
+  printf("\n");
+}
+
